/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.generic;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.apache.log4j.Logger;
import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;

@Stateless
@Remote(CrudManager.class)
public class CrudManagerBean implements CrudManager {
	@PersistenceContext
	EntityManager em;

	protected static Logger logger = Logger.getLogger(CrudManagerBean.class.getName());
	
	public <T> T create(T t) {
		this.em.persist(t);
		this.em.flush();
		this.em.refresh(t);
		return t;
	}

	@SuppressWarnings("unchecked")
	public <T> T find(Object id, Class<T> type) {
		return (T) this.em.find(type, id);
	}

//	public <T> void delete(Object id, Class<T> t) {
//		Object ref = this.em.getReference(t, id);
//		this.em.remove(ref);
//	}
	
	public void delete(Object t) {
		Object ref = this.em.merge(t);
		this.em.remove(ref);
	}

	public <T> T update(T t) {
		return (T) this.em.merge(t);
	}

	public <T>List<T> findByDetachedCriteria(int startRow, int pageSize, DetachedCriteria detachedCriteria) {
		if (detachedCriteria == null)
			return new ArrayList<T>();
		
		Session session = (Session)em.getDelegate();	
		
		/**
		* INFO
		* detachedCriteria contiene il ResultTransformer DISTINCT_ROOT_ENTITY (che raggruppa i risultati per entita definita come root)
		* dato che Hibernate effettua la distinct non a livello query, ma a livello risultati,
		* non è possibile selezionare il set di fatture per la paginazione utilizzando
		* i metodi setMaxResults e setFirstResult (che invece opera a livello query) per ottenere il num. dei record trovati
		* 
		* List results = detachedCriteria.getExecutableCriteria(session).setMaxResults(pageSize).setFirstResult(startRow).list();
		*/
		List allResults = detachedCriteria.getExecutableCriteria(session).list();
		List pagedResults = new ArrayList();
		//range di risultati da visualizzare
		for(int i=startRow; i<startRow+pageSize; i++){
			if(allResults.size()>i){
				pagedResults.add(allResults.get(i));
			}
			else {
				break;
			}
		}
		
		return pagedResults;
	}
	
	public int countByDetachedCriteria(DetachedCriteria detachedCriteria) {
		if (detachedCriteria == null)
			return 0;
		
		Session session = (Session)em.getDelegate();
		
		/**
		* INFO
		* detachedCriteria contiene il ResultTransformer DISTINCT_ROOT_ENTITY (che raggruppa i risultati per entita definita come root)
		* dato che Hibernate effettua la distinct non a livello query, ma a livello risultati,
		* non è possibile utilizzare il metodo rowCount (che invece opera a livello query) per ottenere il num. dei record trovati
		* 
		* List results = detachedCriteria.getExecutableCriteria(session).setProjection( Projections.projectionList().add(Projections.rowCount())).list();
		*/
		List results = detachedCriteria.getExecutableCriteria(session).list();
		
		int out = 0;
		if (results == null || results.size() == 0)
			out = 0;
		else 

			out = (Integer)results.size();
		
		return out;
	}
	
	public <T>List<T> findByCriteria(Class<T> clazz,int startRow, int pageSize, List<Criterion> restrictions, Order orderBy) {
		try {
			Session session = (Session)em.getDelegate();
			Criteria criteria = session.createCriteria(clazz);
			criteria.setMaxResults(pageSize);
			criteria.setFirstResult(startRow);
			if (restrictions != null)
				for (Criterion c : restrictions)
					if (c instanceof NestedCriterion){
						String outerProp = ((NestedCriterion)c).getOuterPropertyName();
						Criterion inner = ((NestedCriterion)c).getInnerCriterion();
						criteria.createCriteria(outerProp).add(inner);
					}
					else
						criteria.add(c);
			
			if (orderBy != null){
				if (orderBy instanceof NestedOrder){
					String outerProp = ((NestedOrder)orderBy).getOuterPropertyName();
					String innerProp = ((NestedOrder)orderBy).getInnerPropertyName();
					boolean asc = ((NestedOrder)orderBy).isAscending();
					criteria.createCriteria(outerProp).addOrder( (asc) ? Order.asc(innerProp) : Order.desc(innerProp));
				}
				else
					criteria.addOrder(orderBy);
			}
			
			return criteria.list();	
		} catch (Exception e) {
			logger.warn("Error calling CrudManagerBean.findByCriteria()", e);
		}
		return new ArrayList();
	}
	
	public int countByCriteria(Class clazz, List<Criterion> restrictions) {
		try {
			org.hibernate.Session session = (Session)em.getDelegate();
			Criteria criteria = session.createCriteria(clazz);
			
			if (restrictions != null)
				for (Criterion c : restrictions)
					if (c instanceof NestedCriterion){
						String outerProp = ((NestedCriterion)c).getOuterPropertyName();
						Criterion inner = ((NestedCriterion)c).getInnerCriterion();
						criteria.createCriteria(outerProp).add(inner);
					}
					else
						criteria.add(c);
			
			criteria.setProjection( Projections.projectionList().add(Projections.rowCount()));
			if (criteria.list() == null || criteria.list().size() == 0)
				return 0;
			else 
				return (Integer)criteria.list().get(0);
		
		} catch (Exception e) {
			logger.warn("Error calling CrudManagerBean.countByCriteria()", e);
		}
		return 0;
	}
	
	
	public <T> List<T> findByQuery(String query) {
		return this.em.createQuery(query).getResultList();
	}

	public <T> List<T> findByQuery(String query,
			Map<String, Object> parameters) {
		return findByQuery(query, parameters, 0);
	}

	public <T> List<T> findByQuery(String query, int resultLimit) {
		return this.em.createQuery(query).setMaxResults(resultLimit)
				.getResultList();
	}

	public <T> List<T> findByQuery(String query,
			Map<String, Object> parameters, int resultLimit) {
		Set<Entry<String, Object>> rawParameters = parameters.entrySet();
		Query q = this.em.createQuery(query);
		if (resultLimit > 0)
			q.setMaxResults(resultLimit);
		for (Entry<String, Object> entry : rawParameters) {
			q.setParameter(entry.getKey(), entry.getValue());
		}
		return q.getResultList();
	}

	public int countItems(String query, Map<String, Object> parameters) {
		Set<Entry<String, Object>> rawParameters = parameters.entrySet();
		Query q = this.em.createQuery(query);
		for (Entry<String, Object> entry : rawParameters) {
			q.setParameter(entry.getKey(), entry.getValue());
		}
		return ((Integer)q.getSingleResult());
	}

	public <T> List<T> findByQuery(String query,
			Map<String, Object> parameters, int startingRow, int resultLimit) {
		Set<Entry<String, Object>> rawParameters = parameters.entrySet();
		Query q = this.em.createQuery(query);
		
		if (startingRow> 0)
			q.setFirstResult(startingRow);
		if (resultLimit > 0)
			q.setMaxResults(resultLimit);
		
		for (Entry<String, Object> entry : rawParameters) {
			q.setParameter(entry.getKey(), entry.getValue());
		}
		return q.getResultList();
	}
	
	public <T> List<T> findByNativeQuery(String query) {
		return this.em.createNativeQuery(query).getResultList();
	}

	public <T> List<T> findByNativeQuery(String nativeQuery, Class<T> resultClass) {
		return this.em.createNativeQuery(nativeQuery, resultClass).getResultList();
	}

	public <T> List<T> findByNativeQuery(String nativeQuery, String resultSetMapping) {
		return this.em.createNativeQuery(nativeQuery, resultSetMapping).getResultList();
	}

	public int executeNativeQuery(String nativeQuery) {
		return this.em.createNativeQuery(nativeQuery).executeUpdate();
	}
}
