/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it, fulvio.dimarco@epocaricerca.it, stefano.monti@epocaricerca.it
 * 
 *
 */
package it.fe.core.ejb.invoice;

import it.fe.core.common.exception.InvalidInvoiceException;
import it.fe.core.ejb.utility.MeterManager;
import it.fe.core.par.invoice.AdditionalData;
import it.fe.core.par.invoice.Client;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.invoice.InvoiceLine;
import it.fe.core.par.invoice.InvoiceState;
import it.fe.core.par.invoice.OriginalDocument;
import it.fe.core.par.invoice.PreviousDocument;
import it.fe.core.par.invoice.Supplier;
import it.fe.core.par.invoice.TaxDetail;
import it.fe.core.par.utility.Meter;
import it.fe.core.par.utility.MeterSummary;
import it.fe.core.par.utility.Reading;
import it.fe.core.par.utility.SupplyInfo;
import it.fe.core.par.utility.UtilityConsumptionPoint;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.ejb.EJB;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityExistsException;
import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceException;
import javax.persistence.Query;
import javax.persistence.TransactionRequiredException;

import org.apache.log4j.Logger;


@Stateless
@Remote(InvoiceManager.class)
public class InvoiceManagerBean implements InvoiceManager {

	@PersistenceContext(name="FatturazioneElettronicaPersistenceUnit")
	EntityManager entityManager;
	
	@EJB
	private SupplierManager supplierManager;
	
	@EJB
	private ClientManager clientManager;
	
	@EJB
	private InvoiceStateManager invoiceStateManager;
	
	@EJB
	private MeterManager meterManager;
	
	@EJB
	private PreviousDocumentManager previousDocumentManager;
	
	protected static Logger logger = Logger.getLogger(InvoiceManagerBean.class.getName());

	
	public Invoice getInvoiceById(long id) {
		Invoice invoice = new Invoice();
		
		try {
			invoice = entityManager.find(Invoice.class, id);
		} catch (NoResultException e) {
			logger.error(e.getMessage());
			invoice = null;
		} catch (NonUniqueResultException e) {
			logger.error(e.getMessage());
			invoice = null;
		} catch (EntityNotFoundException e) {
			logger.error(e.getMessage());
			invoice = null;
		}
		
		return invoice;
	}

	@SuppressWarnings("unchecked")
	public List<Invoice> getInvoices() {
		Query query = entityManager.createQuery("SELECT i FROM Invoice i");
		return query.getResultList();
	}
	
	@SuppressWarnings("unchecked")
	public List<Invoice> getPagedInvoices(int first, int pageSize) {
		logger.info("Lazily loading invoices between "+first+" and "+first+pageSize);
		Query query = entityManager.createQuery("SELECT i FROM Invoice i");
		query.setMaxResults(pageSize) ;
		query.setFirstResult(first) ;
		List<Invoice> invoices =  query.getResultList();
		return invoices;
	}

	public long insertInvoice(Invoice invoice) {
		long result = 0;
		Supplier supplierDb;
		Client clientDb;
		Meter meterDb = null;
		
		String supplierVatId = "";
		String clientVatId = "";
		if (invoice.getSupplier() != null)
			supplierVatId = invoice.getSupplier().getVatId();
		if (invoice.getClient() != null)
			clientVatId = invoice.getClient().getVatId();
		
		//Fornitore
		if ( !supplierVatId.isEmpty()) {
			supplierDb = supplierManager.getSupplierByVatId(supplierVatId);	
			if (supplierDb != null) {
				logger.info("Fornitore già presente nel db");
				invoice.setSupplier(supplierDb);
			}
			else {
				logger.info("Inserimento nuovo fornitore");
				long supplierId = supplierManager.insertSupplier(invoice.getSupplier());
				invoice.getSupplier().setSupplierId(supplierId);
			}
		}
		
		
		//Cliente
		if (!clientVatId.isEmpty()) {
			clientDb = clientManager.getClientByVatId(clientVatId);
			if (clientDb != null) {
				logger.info("Cliente già presente nel db");
				invoice.setClient(clientDb);
			}
			else {
				logger.info("Inserimento nuovo cliente");
				long clientId = clientManager.insertClient(invoice.getClient());
				invoice.getClient().setClientId(clientId);
			}
		}
		
		//Stato della fattura
		if (invoice.getInvoiceState() == null) {
			logger.info("Imposta lo stato fattura a ricevuta");
			invoice.setInvoiceState(invoiceStateManager.getInvoiceStateByDescription("Ricevuta"));
		}
		
		
		//Documenti pregressi
		if(invoice.getPreviousDocuments() != null){
			for(int i=0; i<invoice.getPreviousDocuments().size(); i++){
				PreviousDocument previousDocumentDb;
				if(invoice.getPreviousDocuments().get(i).getDocumentCode() != null) {
					previousDocumentDb = previousDocumentManager.findPreviousDocumentByDocumentCode(invoice.getPreviousDocuments().get(i).getDocumentCode(),
																								invoice.getSupplier());
	
					if (previousDocumentDb != null) {
						logger.info("Trovato il documento " + previousDocumentDb.getDocumentCode());
						invoice.getPreviousDocuments().set(i, previousDocumentDb);
					}
					else {
						logger.info("Inserimento del documento num. " + invoice.getPreviousDocuments().get(i).getDocumentCode());
						//fornitore
						if(invoice.getPreviousDocuments().get(i).getSupplier() == null){
							invoice.getPreviousDocuments().get(i).setSupplier(invoice.getSupplier());
						}
						long previousDocumentId = previousDocumentManager.insertPreviousDocument(invoice.getPreviousDocuments().get(i));
						invoice.getPreviousDocuments().get(i).setPreviousDocumentId(previousDocumentId);
					}
				}
				else{
					logger.error("Documento senza codice");
					invoice.getPreviousDocuments().remove(i);
					
					i--;
				}
			}
		}
		
			
		//Contatore
		for (InvoiceLine invoiceLine : invoice.getInvoiceLines()) {
		
			if(invoiceLine.getMeter() != null) {
				
				//Cerco per numero contatore e numero presa
				if ((invoiceLine.getMeter().getMeterNumber() != null) && (!invoiceLine.getMeter().getMeterNumber().equals(""))
						&& (invoiceLine.getMeter().getUtilityConsumptionPoint() != null)) {
					meterDb = meterManager.findMeter(invoiceLine.getMeter().getMeterNumber(),
							invoiceLine.getMeter().getUtilityConsumptionPoint().getConnectionNumber());
				}
				
				if(meterDb == null){
					//Se non lo trovo cerco per numero contatore e indirizzo
					meterDb = meterManager.findMeter(invoiceLine.getMeter().getMeterNumber(),
							invoiceLine.getMeter().getMeterAddress());
				}
			
				if(meterDb != null){
					logger.info("Contatore già presente nel db");
					
					//Controllo se il contatore è già associato a questo contratto
					if(invoiceLine.getMeter().getSupplyInfo().size() > 0){
						PreviousDocument previousDocumentDb;
						Boolean flgContractOk = false;
						
						for(SupplyInfo supplyInfo : invoiceLine.getMeter().getSupplyInfo()){
							
							for(SupplyInfo supplyInfoMeterDb : meterDb.getSupplyInfo()){
							
								if(supplyInfo.getPreviousDocument() != null){
									if(supplyInfo.getPreviousDocument().getDocumentCode().equals(supplyInfoMeterDb.getPreviousDocument().getDocumentCode())){
										flgContractOk = true;
									}

								}
							}
							
							if(!flgContractOk){
								//aggiungo il nuovo contratto										
								previousDocumentDb = previousDocumentManager.findContractByDocumentCode(supplyInfo.getPreviousDocument().getDocumentCode(),
										invoice.getSupplier());

								if (previousDocumentDb != null) {
									logger.info("Trovato il documento " + previousDocumentDb.getDocumentCode());
									supplyInfo.setPreviousDocument(previousDocumentDb);
								}
								else {
									logger.info("Inserimento del contratto num. " + supplyInfo.getPreviousDocument().getDocumentCode());
									if(supplyInfo.getPreviousDocument().getSupplier() == null){
										supplyInfo.getPreviousDocument().setSupplier(invoice.getSupplier());
									}
									long contractId = previousDocumentManager.insertPreviousDocument(supplyInfo.getPreviousDocument());
									supplyInfo.getPreviousDocument().setPreviousDocumentId(contractId);
								}
								
								meterDb = meterManager.updateMeterSupplyInfo(meterDb.getMeterId(),supplyInfo);
							}
						}
					}					
					
					invoiceLine.setMeter(meterDb);
				}
				else {
					logger.info("Inserimento nuovo contatore");
					
					//fornitore per contratto-contatore
					for(int z=0; z<invoiceLine.getMeter().getSupplyInfo().size(); z++){
						invoiceLine.getMeter().getSupplyInfo().get(z).getPreviousDocument().setSupplier(invoice.getSupplier());
					}
					
					long meterId = meterManager.insertMeter(invoiceLine.getMeter());
					invoiceLine.setMeter(meterManager.getMeterById(meterId));
				}
			}
		}
		
		//Letture
		if(invoice.getReadingList() != null){
			for(Reading reading : invoice.getReadingList()){
				if(reading.getMeter() != null) {
					meterDb = meterManager.findMeter(reading.getMeter().getMeterNumber(),
							reading.getMeter().getMeterAddress());
	
					if(meterDb != null){
						logger.info("Contatore già presente nel db");
						reading.setMeter(meterDb);
					}
					else {
						logger.info("Inserimento nuovo contatore");
						
						//fornitore per contratto-contatore
						for(SupplyInfo supplyInfoMeterReading : reading.getMeter().getSupplyInfo()){
							if(supplyInfoMeterReading.getPreviousDocument().getSupplier() == null){
								supplyInfoMeterReading.getPreviousDocument().setSupplier(invoice.getSupplier());
							}
						}
						
						long meterId = meterManager.insertMeter(reading.getMeter());
						reading.setMeter(meterManager.getMeterById(meterId));
					}
				}
			}
		}
		
		//Riassunto contatore
		if(invoice.getMeterSummary() != null){
			for(MeterSummary meterSummary : invoice.getMeterSummary()){
				if(meterSummary.getMeter() != null) {
					meterDb = meterManager.findMeter(meterSummary.getMeter().getMeterNumber(),
							meterSummary.getMeter().getMeterAddress());
	
					if(meterDb != null){
						logger.info("Contatore già presente nel db");
						meterSummary.setMeter(meterDb);
					}
					else {
						logger.info("Inserimento nuovo contatore");

						//fornitore per contratto-contatore
						for(SupplyInfo supplyInfoMeterSummary : meterSummary.getMeter().getSupplyInfo()){
							if(supplyInfoMeterSummary.getPreviousDocument().getSupplier() == null){
								supplyInfoMeterSummary.getPreviousDocument().setSupplier(invoice.getSupplier());
							}
						}
						long meterId = meterManager.insertMeter(meterSummary.getMeter());
						meterSummary.setMeter(meterManager.getMeterById(meterId));
					}
				}
			}
		}
		

		
		
		try {
			 entityManager.persist(invoice);
			 result = invoice.getInvoiceId();
			 entityManager.flush();
		} catch (EntityExistsException e) {
			logger.error(e.getMessage());
			result = -1;
		} catch (IllegalStateException e) {
			logger.error(e.getMessage());
			result = -1;
		} catch (IllegalArgumentException e) {
			logger.error(e.getMessage());
			result = -1;
		} catch (PersistenceException e) {
			logger.error(e.getMessage());
			result = -1;
		}

		return result;
	}
	
	public int removeInvoiceById(long id) {
		int result;
		try {
			Invoice invoice = entityManager.find(Invoice.class, id);
			entityManager.remove(invoice);
			result = 1;
		} catch (IllegalStateException e) {
			logger.error(e.getMessage());
			result = -1;
		} catch (IllegalArgumentException e) {
			logger.error(e.getMessage());
			result = -1;
		} catch (TransactionRequiredException e) {
			logger.error(e.getMessage());
			result = -1;
		}	
		return result;
	}
	
	public int checkInvoice(Invoice invoice) {
		int check = -1;
		String message;
		
		if (invoice.getIssueDate() == null) {
			message = "The invoice issue date is null";
			logger.error(message);
			throw new InvalidInvoiceException(message);
		}
		else if (invoice.getInvoiceNumber() == null) {
			message = "The invoice number is null";
			logger.error(message);
			throw new InvalidInvoiceException(message);
		}
		else if (invoice.getAllInclusiveTotal() == 0) {
			message = "The invoice all inclusive total is zero";
			logger.error(message);
			throw new InvalidInvoiceException(message);
		}
		else if (invoice.getReceiptDate() == null) {
			message = "The invoice receipt date is null";
			logger.error(message);
			throw new InvalidInvoiceException(message);
		}
		else if (invoice.getClient() == null) {
			message = "The invoice client is null";
			logger.error(message);
			throw new InvalidInvoiceException(message);
		}
		else if (invoice.getSupplier() == null) {
			message = "The invoice supplier is null";
			logger.error(message);
			throw new InvalidInvoiceException(message);
		}
		else if ((invoice.getSupplier().getVatId() == null) || (invoice.getSupplier().getVatId().equals(""))) {
			message = "The invoice supplier vat id is null";
			logger.error(message);
			throw new InvalidInvoiceException(message);
		}
		else {
			message = "The invoice is ok";
			logger.info(message);
			check = 1;
		}
		
		return check;
	}
	
	public Invoice updateProcessInstanceId(long id, String processInstanceId) {
		Invoice invoice = entityManager.find(Invoice.class, id);
		if (invoice != null) {
			invoice.setProcessInstanceId(processInstanceId);
		}
		return invoice;
	}
	
	public int findTotalInvoices() {
		Query query = entityManager.createQuery("SELECT COUNT(i) FROM Invoice i");
		Long result = (Long) query.getSingleResult();
		return result.intValue();
	}
	
	
	public Invoice findInvoiceByNumberAndVatId(String invoiceNumber, String supplierVatId) {
		Invoice invoice;
		Long result;
		
		Query query = entityManager.createQuery("SELECT i FROM Invoice i WHERE (i.invoiceNumber = ?1 AND i.supplier.vatId = ?2)");
		query.setParameter(1, invoiceNumber);
		query.setParameter(2, supplierVatId);
		
		try {
			invoice = (Invoice) query.getSingleResult();
			result = invoice.getInvoiceId();
			logger.info("Find invoice number " + invoiceNumber + "(id = " + result + ")");
		} catch (NoResultException e) {
			logger.error(e.getMessage());
			invoice = null;
		} catch (NonUniqueResultException e) {
			logger.error(e.getMessage());
			invoice = null;
		} catch (EntityNotFoundException e) {
			logger.error(e.getMessage());
			invoice = null;
		}
		
		return invoice;
	}
	
	
	public Invoice updateInvoiceState(long id, InvoiceState state) {
		Invoice invoice = entityManager.find(Invoice.class, id);
		if (invoice != null) {
			invoice.setInvoiceState(state);
		}
		return invoice;
	}
	
	public Invoice updateInvoiceErrorState(long id, boolean isInError, String errorMessage) {
		Invoice invoice = this.getInvoiceById(id);
		if(invoice != null) {
			invoice.setError(isInError);
			invoice.setErrorMessage(errorMessage);
		}
		return invoice;
	}
	
	public List<PreviousDocument> fetchPreviousDocuments(Invoice invoice) {
		invoice = entityManager.merge(invoice);
		invoice.getPreviousDocuments().size();
		List<PreviousDocument> attachedList = invoice.getPreviousDocuments();
		List<PreviousDocument> detachedList = new ArrayList<PreviousDocument>();
		for (PreviousDocument pd : attachedList) {
			detachedList.add(pd);
		}
		//invoice.setPreviousDocuments(detachedList);
		return detachedList;
	}
	
	public List<InvoiceLine> fetchInvoiceLines(Invoice invoice) {
		invoice = entityManager.merge(invoice);
		invoice.getInvoiceLines().size();
		List<InvoiceLine> attachedList = invoice.getInvoiceLines();
		List<InvoiceLine> detachedList = new ArrayList<InvoiceLine>();
		for (InvoiceLine il : attachedList) {
			detachedList.add(il);
		}
		//invoice.setInvoiceLines(detachedList);
		return detachedList;
	}
	
	public List<Object[]> fetchInvoiceLineAccountingCosts(Invoice invoice) {
		List<Object[]> resultList = entityManager.createQuery("select distinct lineAccountingCost,currencyID, sum(lineExtensionAmount) from InvoiceLine l where l.invoice=:invoice group by lineAccountingCost, currencyID").setParameter("invoice", invoice).getResultList();
		return resultList ;
	}
	
	public List<MeterSummary> fetchMeterSummaries(Invoice invoice) {
		invoice = entityManager.merge(invoice);
		invoice.getMeterSummary().size();
		List<MeterSummary> attachedList = invoice.getMeterSummary();
		List<MeterSummary> detachedList = new ArrayList<MeterSummary>();
		for (MeterSummary ms : attachedList) {
			detachedList.add(ms);
		}
		//invoice.setMeterSummary(detachedList);
		return detachedList;
	}
	
	public List<AdditionalData> fetchInvoiceAdditionalData(Invoice invoice) {
		invoice = entityManager.merge(invoice);
		invoice.getAdditionalData().size();
		List<AdditionalData> attachedList = invoice.getAdditionalData();
		List<AdditionalData> detachedList = new ArrayList<AdditionalData>();
		for (AdditionalData ad : attachedList) {
			detachedList.add(ad);
		}
		//invoice.setAdditionalData(detachedList);
		return detachedList;
	}
	
	public List<InvoiceLine> fetchInvoiceLinesWithAdditionalData(Invoice invoice) {
		invoice = entityManager.merge(invoice);
		invoice.getInvoiceLines().size();
		List<InvoiceLine> invoiceLinesAttached = invoice.getInvoiceLines();
		List<InvoiceLine> invoiceLinesDetached = new ArrayList<InvoiceLine>();
		for(InvoiceLine il : invoiceLinesAttached) {
			il.getAdditionalData().size();
			List<AdditionalData> additionalDataAttached = il.getAdditionalData();
			List<AdditionalData> additionalDataDetached = new ArrayList<AdditionalData>();
			for (AdditionalData ad : additionalDataAttached) {
				additionalDataDetached.add(ad);
			}
			il.setAdditionalData(additionalDataDetached);
			invoiceLinesDetached.add(il);
		}
		return invoiceLinesDetached;
	}
	
	public List<AdditionalData> fetchInvoiceLineAdditionalData(InvoiceLine invoiceLine) {
		invoiceLine = entityManager.merge(invoiceLine);
		invoiceLine.getAdditionalData().size();
		List<AdditionalData> additionalDataAttached = invoiceLine.getAdditionalData();
		List<AdditionalData> additionalDataDetached = new ArrayList<AdditionalData>();
		for (AdditionalData ad : additionalDataAttached) {
			additionalDataDetached.add(ad);
		}
		return additionalDataDetached;
	}
 
	public Invoice fetchXMLFile(Invoice invoice) {
		invoice = entityManager.merge(invoice);
		invoice.setXmlFile(invoice.getXmlFile());
		
		return invoice;
	}
	
	public List<TaxDetail> fetchTaxDetails(Invoice invoice) {
		invoice = entityManager.merge(invoice);

		List<TaxDetail> taxDetailAttached = invoice.getTaxDetails();
		List<TaxDetail> taxDetailDetached = new ArrayList<TaxDetail>();
		for (TaxDetail td : taxDetailAttached) {
			taxDetailDetached.add(td);
		}
		return taxDetailDetached;
	}
	
	public List<Reading> fetchReadings(Invoice invoice) {
		invoice = entityManager.merge(invoice);

		List<Reading> readingAttached = invoice.getReadingList();
		List<Reading> readingDetached = new ArrayList<Reading>();
		for (Reading td : readingAttached) {
			readingDetached.add(td);
		}
		return readingDetached;
	}
	
	public boolean invoiceExists(long id) {
		String q = "SELECT EXISTS (SELECT * FROM Invoices WHERE invoiceId ="+id+")";
		Query query = entityManager.createNativeQuery(q);
		Number result = (Number) query.getSingleResult();
		if (result.intValue() == 1) 
			return true;
		else
			return false;
	}
	
	
	//--------------- @EJB Getters/Setters --------------------------
	
	public SupplierManager getSupplierManager() {
		return supplierManager;
	}

	public void setSupplierManager(SupplierManager supplierManager) {
		this.supplierManager = supplierManager;
	}

	public ClientManager getClientManager() {
		return clientManager;
	}

	public void setClientManager(ClientManager clientManager) {
		this.clientManager = clientManager;
	}

	public InvoiceStateManager getInvoiceStateManager() {
		return invoiceStateManager;
	}

	public void setInvoiceStateManager(InvoiceStateManager invoiceStateManager) {
		this.invoiceStateManager = invoiceStateManager;
	}

	public MeterManager getMeterManager() {
		return meterManager;
	}

	public void setMeterManager(MeterManager meterManager) {
		this.meterManager = meterManager;
	}

	public PreviousDocumentManager getPreviousDocumentManager() {
		return previousDocumentManager;
	}

	public void setPreviousDocumentManager(
			PreviousDocumentManager previousDocumentManager) {
		this.previousDocumentManager = previousDocumentManager;
	}

	public List<AdditionalData> fetchAdditionalData(Invoice invoice) {
		// TODO Auto-generated method stub
		return null;
	}

	public Invoice updateInvoice(Invoice invoice) {
		Invoice managedInvoice=entityManager.merge(invoice);
		return managedInvoice;
	}

	@Override
	public List<OriginalDocument> fetchOriginalDocuments(Invoice invoice) {
		invoice = entityManager.merge(invoice);
		invoice.getOriginalDocuments().size();
		List<OriginalDocument> attachedList = invoice.getOriginalDocuments();
		List<OriginalDocument> detachedList = new ArrayList<OriginalDocument>();
		for (OriginalDocument pd : attachedList) {
			detachedList.add(pd);
		}
		return detachedList;
	}

	

	
	
}
