/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.utility;

import it.fe.core.ejb.invoice.PreviousDocumentManager;
import it.fe.core.par.common.PostalAddress;
import it.fe.core.par.invoice.PreviousDocument;
import it.fe.core.par.utility.Meter;
import it.fe.core.par.utility.Sector;
import it.fe.core.par.utility.SpecificSector;
import it.fe.core.par.utility.SupplyInfo;

import java.util.Date;
import java.util.List;

import javax.ejb.EJB;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityExistsException;
import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceException;
import javax.persistence.Query;
import javax.persistence.TransactionRequiredException;

import org.apache.log4j.Logger;

@Stateless
@Remote(MeterManager.class)
public class MeterManagerBean implements MeterManager {

	@PersistenceContext(name="FatturazioneElettronicaPersistenceUnit")
	EntityManager entityManager;
	
	@EJB
	public SectorManager sectorManager;
	
	@EJB
	public PreviousDocumentManager previousDocumentManager;
	
	@EJB
	public SpecificSectorManager specificSectorManager;
	
	protected static Logger logger = Logger.getLogger(MeterManagerBean.class.getName());
	
	
	public Meter getMeterById(long id) {
		Meter meter = new Meter();
		
		try {
			meter = entityManager.find(Meter.class, id);
		} catch (NoResultException e) {
			logger.error(e.getMessage());
			meter = null;
		} catch (NonUniqueResultException e) {
			logger.error(e.getMessage());
			meter = null;
		} catch (EntityNotFoundException e) {
			logger.error(e.getMessage());
			meter = null;
		}
		
		return meter;
	}

	@SuppressWarnings("unchecked")
	public List<Meter> getMeters() {
		Query query = entityManager.createQuery("SELECT m FROM Meter m");
		return query.getResultList();
	}
	
	public Meter findMeter(String meterNumber, PostalAddress meterAddress){
		
		if (meterAddress == null){
			logger.error("Meter Address null, impossibile effettuare la riciesta");
			return null;
		}
		
		Meter meter;
		
		Query query = entityManager.createQuery("SELECT m FROM Meter m WHERE (m.meterNumber = ?1 " +
																				"AND m.meterAddress.streetName = ?2 " +
																				"AND m.meterAddress.buildingNumber = ?3 " +
																				"AND m.meterAddress.cityName = ?4)");
		query.setParameter(1, meterNumber);
		query.setParameter(2, meterAddress.getStreetName());
		query.setParameter(3, meterAddress.getBuildingNumber());
		query.setParameter(4, meterAddress.getCityName());
		
		try {			
			meter = (Meter) query.getSingleResult();
			//logger.info("Find meter by " + meterNumber + " and " + meterAddress);
		} catch (NoResultException e) {
			logger.error(e.getMessage());
			meter = null;
		} catch (NonUniqueResultException e) {
			logger.error(e.getMessage());
			meter = null;
		} catch (EntityNotFoundException e) {
			logger.error(e.getMessage());
			meter = null;
		}
		
		return meter;
	}
	
	public Meter findMeter(String meterNumber, String connectionNumber){
		
		Meter meter;
		
		Query query = entityManager.createQuery("SELECT m FROM Meter m WHERE m.meterNumber = ?1 " +
																	  "AND m.utilityConsumptionPoint.connectionNumber = ?2 ");
		query.setParameter(1, meterNumber);
		query.setParameter(2, connectionNumber);
		
		try {			
			meter = (Meter) query.getSingleResult();
			//logger.info("Find meter by " + meterNumber + " and " + meterAddress);
		} catch (NoResultException e) {
			logger.error(e.getMessage());
			meter = null;
		} catch (NonUniqueResultException e) {
			logger.error(e.getMessage());
			meter = null;
		} catch (EntityNotFoundException e) {
			logger.error(e.getMessage());
			meter = null;
		}
		
		return meter;
	}

	public long insertMeter(Meter meter) {
		long result = 0;
		Sector sectorDb;
		
		meter.setAction("INS");
		
		//Se il contatore viene inserito in modo automatico dal sistema, non c'è l'identificativo dell'utente
		if((meter.getRegistrationUser() == null) || (meter.getRegistrationUser().equals(""))) {
			meter.setRegistrationUser(Meter.RegistrationUserType.AUTOMATICO.toString());
		}
		if(meter.getRegistrationDate() == null) {
			meter.setRegistrationDate(new Date());
		}
		
		//Se il contatore viene inserito in modo automatico dal sistema non c'è il sectorId, ma solo il sectorName
		if(meter.getSector() != null) {
			if(meter.getSector().getSectorId() != null){
				sectorDb = sectorManager.findSectorById(meter.getSector().getSectorId());
				if (sectorDb != null) {
					logger.info("Trovato il settore merceologico " + sectorDb.getSectorName());
					meter.setSector(sectorDb);
				}
				else {
					logger.error("Impossibile trovare il settore merceologico!");
				}
			}
			else {
				sectorDb = sectorManager.findSectorByName(meter.getSector().getSectorName());
				if (sectorDb != null) {
					logger.info("Trovato il settore merceologico " + sectorDb.getSectorName());
					meter.setSector(sectorDb);
				}
				else {
					logger.error("Impossibile trovare il settore merceologico!");
				}
			}
		}
		else {
			logger.error("Impossibile trovare il settore merceologico!");
		}
		
		//Controllo info fornitura
		if(meter.getSupplyInfo().size() > 0){
			PreviousDocument previousDocumentDb;
			SpecificSector specificSectorDb;
			for(int i=0; i<meter.getSupplyInfo().size(); i++){
				
				//Contratto
				if(meter.getSupplyInfo().get(i).getPreviousDocument() != null){
					previousDocumentDb = previousDocumentManager.findContractByDocumentCode(meter.getSupplyInfo().get(i).getPreviousDocument().getDocumentCode(),
																			meter.getSupplyInfo().get(i).getPreviousDocument().getSupplier());
				
					if (previousDocumentDb != null) {
						logger.info("Trovato il documento " + previousDocumentDb.getDocumentCode());
						meter.getSupplyInfo().get(i).setPreviousDocument(previousDocumentDb);
					}
					else {
						logger.error("Inserimento del contratto num. " + meter.getSupplyInfo().get(i).getPreviousDocument().getDocumentCode());
						long contractId = previousDocumentManager.insertPreviousDocument(meter.getSupplyInfo().get(i).getPreviousDocument());
						meter.getSupplyInfo().get(i).getPreviousDocument().setPreviousDocumentId(contractId);
					}			
				}
				
				//fornitura
				if(meter.getSupplyInfo().get(i).getSpecificSector() != null){
					specificSectorDb = specificSectorManager.findSpecificSectorByName(meter.getSupplyInfo().get(i).getSpecificSector().getSpecificSectorName());
					if (specificSectorDb != null) {
						logger.info("Trovata la fornitura " + specificSectorDb.getSpecificSectorName());
						meter.getSupplyInfo().get(i).setSpecificSector(specificSectorDb);
					}
					else {
						logger.error("Impossibile trovare la fornitura!");
					}
				}
			}
		}
		
		//Controllo fornitura
		
		
		try {
			 entityManager.persist(meter);
			 result = meter.getMeterId();
			 entityManager.flush();
		} catch (EntityExistsException e) {
			logger.error(e.getMessage());
			result = -1;
		} catch (IllegalStateException e) {
			logger.error(e.getMessage());
			result = -1;
		} catch (IllegalArgumentException e) {
			logger.error(e.getMessage());
			result = -1;
		} catch (PersistenceException e) {
			logger.error(e.getMessage());
			result = -1;
		}

		return result;
	}
	
	public Meter updateMeterSupplyInfo(Long meterId, SupplyInfo supplyInfo){
		Meter meter = entityManager.find(Meter.class, meterId);
		if (meter != null){
			meter.getSupplyInfo().add(supplyInfo);
			meter.setSupplyInfo(meter.getSupplyInfo());
		}
		return meter;
	}

	public int removeMeterById(long id) {
		int result;
		try {
			Meter meter = entityManager.find(Meter.class, id);
			entityManager.remove(meter);
			result = 1;
		} catch (IllegalStateException e) {
			logger.error(e.getMessage());
			result = -1;
		} catch (IllegalArgumentException e) {
			logger.error(e.getMessage());
			result = -1;
		} catch (TransactionRequiredException e) {
			logger.error(e.getMessage());
			result = -1;
		}	
		return result;
	}

	
	//--------------- @EJB Getters/Setters --------------------------
	
	public SectorManager getSectorManager() {
		return sectorManager;
	}

	public void setSectorManager(SectorManager sectorManager) {
		this.sectorManager = sectorManager;
	}

	public PreviousDocumentManager getPreviousDocumentManager() {
		return previousDocumentManager;
	}

	public void setPreviousDocumentManager(
			PreviousDocumentManager previousDocumentManager) {
		this.previousDocumentManager = previousDocumentManager;
	}

	public SpecificSectorManager getSpecificSectorManager() {
		return specificSectorManager;
	}

	public void setSpecificSectorManager(SpecificSectorManager specificSectorManager) {
		this.specificSectorManager = specificSectorManager;
	}

}
