/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 * <br /><br />
* 
 *
 */
package it.fe.core.par.invoice;

import it.fe.core.par.utility.Consumption;
import it.fe.core.par.utility.Meter;

import java.io.Serializable;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;


/**
 * La classe <b><i>InvoiceLine</i></b> riporta ogni voce della fattura legata ad un importo (quantità, prezzo unitario, importo, descrizione della voce, …).
 * <br />La gestione dei campi not null a livello normativo è delegata alla gestione delle eccezioni.
 * 
 * <br /><br />
 * 
 * <b>N.B.</b> Sono presenti le annotazioni Hibernate/JPA per la persistenza sul database di FE.
 *
 */
@Entity
@Table(name="InvoiceLines")
public class InvoiceLine implements Serializable{
	
	private static final long serialVersionUID = 1L;

	/**
	 * Numero progressivo assegnato alla riga fattura.
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private long invoiceLineId;
	
	/**
	 * Numero della riga di dettaglio all'interno della fattura; 
	 * elemento obbligatorio nello standard UBL 2.0.
	 */
	@Column(nullable=false)
	private int lineNumber;

	/**
	 * Quantità del bene specificato nella riga.
	 */
	private double invoicedQuantity;
	
	/**
	 * Unità di misura del bene specificato nella riga.
	 */
	private String unitCode;
	
	/**
	 * Nome del bene/servizio.
	 */
	private String itemName; 
	
	/**
	 * Descrizione del bene specificato nella riga; 
	 * la normativa richiede una descrizione che chiarisca natura e qualità del bene.
	 */
	private String itemDescription; 
	
	/**
	 * Codice del bene assegnato dal fornitore.
	 */
	private String sellersItemIdentification;
	
	/**
	 * Codice Conto Contabile dell'acquirente applicato alla riga fattura.
	 */
	private String lineAccountingCost;
	
	/**
	 * Riferimento alla riga dell'ordine.
	 */
	private String orderLineReference;
	
	/**
	 * Valore normale: costo regolare del bene specificato nella riga.
	 */
	private double priceAmount;
	
	/**
	 * Valuta importo unitario.
	 */
	private String priceAmountCurrencyId;
	
	/**
	 * Quantita base a cui si riferisce priceAmount (es. euro/anno).
	 */
	private double baseQuantity;
	
	/**
	 * Unita di misura della quantita base.
	 */
	private String baseQuantityUnitCode;
	
	/**
	 * Riferimenti a sconti, premi, abbuoni.
	 */
	@OneToMany( fetch=FetchType.LAZY, cascade=CascadeType.ALL )
	@JoinColumn(name="invoiceLineId", referencedColumnName="invoiceLineId")
	private List<AllowanceCharge> allowanceCharges;

	/**
	 * Riferimento all'iva applicata alla riga.
	 */
	@OneToOne( fetch=FetchType.EAGER, cascade=CascadeType.ALL )
	@JoinColumn(name="taxDetailId", referencedColumnName="taxDetailId")
	private TaxDetail taxDetail;
	
	/**
	 * Ammontare netto, comprensivo di sconti, premi, abbuoni, del bene specificato nella riga.
	 */
	@Column(nullable=false)
	private double lineExtensionAmount;

	/**
	 * Valuta.
	 */
	private String currencyID;
	
	/**
	 * Note.
	 */
	@OneToMany( fetch=FetchType.LAZY, cascade=CascadeType.ALL )
	@JoinColumn(name="invoiceLineId", referencedColumnName="invoiceLineId")
	private List<InvoiceLineNote> noteList;

	@ManyToOne( fetch=FetchType.EAGER )
	@JoinColumn(name="previousDocumentId", referencedColumnName="previousDocumentId")
	private PreviousDocument previousDocument;
	
	@ManyToOne
	@JoinColumn(name="invoiceId", referencedColumnName="invoiceId")
	private Invoice invoice;
	
	
	/*** UTILITY INVOICES ***/
	
	/**
	 * Contatore
	 */
	@ManyToOne( fetch=FetchType.EAGER )
	@JoinColumn(name="meterId", referencedColumnName="meterId")
	private Meter meter;
	
	/**
	 * Consumo
	 */
	@OneToOne( fetch=FetchType.EAGER, cascade=CascadeType.ALL )
	@JoinColumn(name="consumptionId", referencedColumnName="consumptionId")
	private Consumption consumption;
	
	@ManyToMany(cascade=CascadeType.ALL, fetch=FetchType.LAZY)
	@JoinTable(name = "InvoiceLines_AdditionalData", joinColumns = @JoinColumn(name = "invoiceLineId"), inverseJoinColumns = @JoinColumn(name = "additionalDataId"))
	private List<AdditionalData> additionalData;
	
	/**
	 * Riferimento al conto contrattuale
	 */
	@Column(nullable=true)
	private String contoContrattuale;
	
	//--------------- Ctors --------------------------
	
	public InvoiceLine() {
		super();
	}

	/**
	 * @param lineNumber
	 * @param invoicedQuantity
	 * @param unitCode
	 * @param itemDescription
	 * @param priceAmount
	 * @param allowanceCharges
	 * @param taxDetail
	 * @param lineExtensionAmount
	 * @param currencyID
	 */
	public InvoiceLine(int lineNumber, double invoicedQuantity,
			String unitCode, String itemDescription, double priceAmount,
			List<AllowanceCharge> allowanceCharges, TaxDetail taxDetail,
			double lineExtensionAmount, String currencyID) {
		super();
		this.lineNumber = lineNumber;
		this.invoicedQuantity = invoicedQuantity;
		this.unitCode = unitCode;
		this.itemDescription = itemDescription;
		this.priceAmount = priceAmount;
		this.allowanceCharges = allowanceCharges;
		this.lineExtensionAmount = lineExtensionAmount;
		this.currencyID = currencyID;
	}
	
	/**
	 * Riga senza sconto
	 * 
	 * @param lineNumber
	 * @param invoicedQuantity
	 * @param unitCode
	 * @param itemDescription
	 * @param priceAmount
	 * @param lineExtensionAmount
	 * @param currencyID
	 */
	public InvoiceLine(int lineNumber, double invoicedQuantity,
			String unitCode, String itemDescription, double priceAmount,
			double lineExtensionAmount, String currencyID) {
		super();
		this.lineNumber = lineNumber;
		this.invoicedQuantity = invoicedQuantity;
		this.unitCode = unitCode;
		this.itemDescription = itemDescription;
		this.priceAmount = priceAmount;
		this.lineExtensionAmount = lineExtensionAmount;
		this.currencyID = currencyID;
	}


	//--------------- Getters/Setters --------------------------

	/**
	 * @return the invoiceLineId
	 */
	public long getInvoiceLineId() {
		return invoiceLineId;
	}


	/**
	 * @param invoiceLineId the invoiceLineId to set
	 */
	public void setInvoiceLineId(long invoiceLineId) {
		this.invoiceLineId = invoiceLineId;
	}


	/**
	 * @return the lineNumber
	 */
	public int getLineNumber() {
		return lineNumber;
	}


	/**
	 * @param lineNumber the lineNumber to set
	 */
	public void setLineNumber(int lineNumber) {
		this.lineNumber = lineNumber;
	}


	/**
	 * @return the invoicedQuantity
	 */
	public double getInvoicedQuantity() {
		return invoicedQuantity;
	}


	/**
	 * @param invoicedQuantity the invoicedQuantity to set
	 */
	public void setInvoicedQuantity(double invoicedQuantity) {
		this.invoicedQuantity = invoicedQuantity;
	}


	/**
	 * @return the unitCode
	 */
	public String getUnitCode() {
		return unitCode;
	}


	/**
	 * @param unitCode the unitCode to set
	 */
	public void setUnitCode(String unitCode) {
		this.unitCode = unitCode;
	}


	/**
	 * @param itemName the itemName to set
	 */
	public void setItemName(String itemName) {
		this.itemName = itemName;
	}

	/**
	 * @return the itemName
	 */
	public String getItemName() {
		return itemName;
	}

	/**
	 * @return the itemDescription
	 */
	public String getItemDescription() {
		return itemDescription;
	}


	/**
	 * @param itemDescription the itemDescription to set
	 */
	public void setItemDescription(String itemDescription) {
		this.itemDescription = itemDescription;
	}


	/**
	 * @param sellersItemIdentification the sellersItemIdentification to set
	 */
	public void setSellersItemIdentification(String sellersItemIdentification) {
		this.sellersItemIdentification = sellersItemIdentification;
	}

	/**
	 * @return the sellersItemIdentification
	 */
	public String getSellersItemIdentification() {
		return sellersItemIdentification;
	}

	/**
	 * @param lineAccountingCost the lineAccountingCost to set
	 */
	public void setLineAccountingCost(String lineAccountingCost) {
		this.lineAccountingCost = lineAccountingCost;
	}

	/**
	 * @return the lineAccountingCost
	 */
	public String getLineAccountingCost() {
		return lineAccountingCost;
	}

	/**
	 * @param orderLineReference the orderLineReference to set
	 */
	public void setOrderLineReference(String orderLineReference) {
		this.orderLineReference = orderLineReference;
	}

	/**
	 * @return the orderLineReference
	 */
	public String getOrderLineReference() {
		return orderLineReference;
	}

	/**
	 * @return the priceAmount
	 */
	public double getPriceAmount() {
		return priceAmount;
	}


	/**
	 * @param priceAmount the priceAmount to set
	 */
	public void setPriceAmount(double priceAmount) {
		this.priceAmount = priceAmount;
	}


	/**
	 * @return the allowanceCharges
	 */
	public List<AllowanceCharge> getAllowanceCharges() {
		return allowanceCharges;
	}


	/**
	 * @param allowanceCharges the allowanceCharges to set
	 */
	public void setAllowanceCharges(List<AllowanceCharge> allowanceCharges) {
		this.allowanceCharges = allowanceCharges;
	}


	/**
	 * @return the lineExtensionAmount
	 */
	public double getLineExtensionAmount() {
		return lineExtensionAmount;
	}


	/**
	 * @param lineExtensionAmount the lineExtensionAmount to set
	 */
	public void setLineExtensionAmount(double lineExtensionAmount) {
		this.lineExtensionAmount = lineExtensionAmount;
	}


	/**
	 * @param taxDetail the taxDetail to set
	 */
	public void setTaxDetail(TaxDetail taxDetail) {
		this.taxDetail = taxDetail;
	}

	/**
	 * @return the taxDetail
	 */
	public TaxDetail getTaxDetail() {
		return taxDetail;
	}

	/**
	 * @return the currencyID
	 */
	public String getCurrencyID() {
		return currencyID;
	}


	/**
	 * @param currencyID the currencyID to set
	 */
	public void setCurrencyID(String currencyID) {
		this.currencyID = currencyID;
	}

	/**
	 * @param noteList the noteList to set
	 */
	public void setNoteList(List<InvoiceLineNote> noteList) {
		this.noteList = noteList;
	}

	/**
	 * @return the noteList
	 */
	public List<InvoiceLineNote> getNoteList() {
		return noteList;
	}

	/**
	 * @param previousDocument the previousDocument to set
	 */
	public void setPreviousDocument(PreviousDocument previousDocument) {
		this.previousDocument = previousDocument;
	}

	/**
	 * @return the previousDocument
	 */
	public PreviousDocument getPreviousDocument() {
		return previousDocument;
	}

	/**
	 * @param invoice the invoice to set
	 */
	public void setInvoice(Invoice invoice) {
		this.invoice = invoice;
	}

	/**
	 * @return the invoice
	 */
	public Invoice getInvoice() {
		return invoice;
	}
	
	/**
	 * @param meter the meter to set
	 */
	public void setMeter(Meter meter) {
		this.meter = meter;
	}


	/**
	 * @return the meter
	 */
	public Meter getMeter() {
		return meter;
	}

	
	/**
	 * @param consumption the consumption to set
	 */
	public void setConsumption(Consumption consumption) {
		this.consumption = consumption;
	}

	/**
	 * @return the consumption
	 */
	public Consumption getConsumption() {
		return consumption;
	}

	public void setAdditionalData(List<AdditionalData> additionalData) {
		this.additionalData = additionalData;
	}

	public List<AdditionalData> getAdditionalData() {
		return additionalData;
	}

	public void setContoContrattuale(String contoContrattuale) {
		this.contoContrattuale = contoContrattuale;
	}

	public String getContoContrattuale() {
		return contoContrattuale;
	}

	/**
	 * @return the baseQuantity
	 */
	public double getBaseQuantity() {
		return baseQuantity;
	}

	/**
	 * @param baseQuantity the baseQuantity to set
	 */
	public void setBaseQuantity(double baseQuantity) {
		this.baseQuantity = baseQuantity;
	}

	/**
	 * @return the baseQuantityUnitCode
	 */
	public String getBaseQuantityUnitCode() {
		return baseQuantityUnitCode;
	}

	/**
	 * @param baseQuantityUnitCode the baseQuantityUnitCode to set
	 */
	public void setBaseQuantityUnitCode(String baseQuantityUnitCode) {
		this.baseQuantityUnitCode = baseQuantityUnitCode;
	}

	/**
	 * @return the priceAmountCurrencyId
	 */
	public String getPriceAmountCurrencyId() {
		return priceAmountCurrencyId;
	}

	/**
	 * @param priceAmountCurrencyId the priceAmountCurrencyId to set
	 */
	public void setPriceAmountCurrencyId(String priceAmountCurrencyId) {
		this.priceAmountCurrencyId = priceAmountCurrencyId;
	}

}
