/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 * <br /><br />
* 
 *
 */
package it.fe.core.par.invoice;

import it.fe.core.par.common.PostalAddress;

import java.io.Serializable;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.AttributeOverrides;
import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.Table;

/**
 * La classe <b><i>Supplier</i></b> contiene le informazioni, necessare in fattura, relative al cliente.
 * 
 * <br /><br />
 * 
 * <b>N.B.</b> Sono presenti le annotazioni Hibernate/JPA per la persistenza sul database di FE.
 *
 */
@Entity
@Table(name="Suppliers")
public class Supplier implements Serializable {
	
	private static final long serialVersionUID = 1L;
	
	/**
	 * Identificativo.
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private Long supplierId;
	
	
	/**
	 * Ragione sociale: per le persone fisiche Nome e Cognome; per le persone giuridiche Denominazione della Ditta.
	 */
	@Column(nullable=false)
	private String supplierName;
	
	
	/**
	 * Il VAT ID Europeo è un codice alfanumerico unico rilasciato da ogni nazione ai propri cittadini ed organizzazioni.
	 * In Italia per le persone fisiche si fa riferimento al Codice Fiscale, per le persone giuridiche alla Partita IVA.
	 */
	@Column(nullable=false)
	private String vatId;
	
	/**
	 * Codice fiscale.
	 */
	@Column
	private String italianFiscalCode;
	
	
	/**
	 * Indirizzo fornitore.
	 * 
	 */
	@Embedded
	private PostalAddress postalAddress;
	
	
	/**
	 * Indirizzo della sede legale del fornitore.
	 * 
	 */
	@Embedded
	@AttributeOverrides( {
		@AttributeOverride(name = "postBox",
				column = @Column(name="registrationAddress_postBox") ),
				@AttributeOverride(name = "streetName",
				column = @Column(name="registrationAddress_streetName") ),
				@AttributeOverride(name = "floor",
				column = @Column(name="registrationAddress_floor") ),
				@AttributeOverride(name = "room",
				column = @Column(name="registrationAddress_room") ),
				@AttributeOverride(name = "additionalStreetName",
				column = @Column(name="registrationAddress_additionalStreetName") ),
				@AttributeOverride(name = "buildingNumber",
				column = @Column(name="registrationAddress_buildingNumber") ),
				@AttributeOverride(name = "department",
				column = @Column(name="registrationAddress_department") ),
				@AttributeOverride(name = "cityName",
				column = @Column(name="registrationAddress_cityName") ),
				@AttributeOverride(name = "postalZone",
				column = @Column(name="registrationAddress_postalZone") ),
				@AttributeOverride(name = "countrySubentityCode",
				column = @Column(name="registrationAddress_countrySubentityCode") ),
				@AttributeOverride(name = "country",
				column = @Column(name="registrationAddress_country")),
				@AttributeOverride(name = "countryName",
				column = @Column(name="registrationAddress_countryName"))
		})
	private PostalAddress registrationAddress;
	
	
	/**
	 * Identifica il punto di arrivo di un servizio di instradamento (es. Numero Ubicazione EAN, GLN, IBAN...).
	 */
	private String endpointId;
	
	
	/**
	 * Lista delle fatture emesse dal fornitore.
	 */
	@OneToMany(	mappedBy="supplier", 
			fetch=FetchType.LAZY)
	private List<Invoice> invoices;


	//--------------- Ctors --------------------------
	
	public Supplier() {
		super();
	}

	/**
	 * @param supplierName
	 * @param vatId
	 * @param postalAddress
	 */
	public Supplier(String supplierName, String vatId,
			PostalAddress postalAddress) {
		super();
		this.supplierName = supplierName;
		this.vatId = vatId;
		this.postalAddress = postalAddress;
	}

	
	//--------------- Getters/Setters --------------------------
	
	/**
	 * @return the supplierId
	 */
	public Long getSupplierId() {
		return supplierId;
	}

	/**
	 * @param supplierId the supplierId to set
	 */
	public void setSupplierId(Long supplierId) {
		this.supplierId = supplierId;
	}

	/**
	 * @return the supplierName
	 */
	public String getSupplierName() {
		return supplierName;
	}

	/**
	 * @param supplierName the supplierName to set
	 */
	public void setSupplierName(String supplierName) {
		this.supplierName = supplierName;
	}

	/**
	 * @return the vatId
	 */
	public String getVatId() {
		return vatId;
	}

	/**
	 * @param vatId the vatId to set
	 */
	public void setVatId(String vatId) {
		this.vatId = vatId;
	}

	/**
	 * @param italianFiscalCode the italianFiscalCode to set
	 */
	public void setItalianFiscalCode(String italianFiscalCode) {
		this.italianFiscalCode = italianFiscalCode;
	}

	/**
	 * @return the italianFiscalCode
	 */
	public String getItalianFiscalCode() {
		return italianFiscalCode;
	}

	/**
	 * @return the postalAddress
	 */
	public PostalAddress getPostalAddress() {
		return postalAddress;
	}

	/**
	 * @param postalAddress the postalAddress to set
	 */
	public void setPostalAddress(PostalAddress postalAddress) {
		this.postalAddress = postalAddress;
	}

	/**
	 * @param registrationAddress the registrationAddress to set
	 */
	public void setRegistrationAddress(PostalAddress registrationAddress) {
		this.registrationAddress = registrationAddress;
	}

	/**
	 * @return the registrationAddress
	 */
	public PostalAddress getRegistrationAddress() {
		return registrationAddress;
	}

	/**
	 * @param endPointId the endPointID to set
	 */
	public void setEndpointId(String endpointId) {
		this.endpointId = endpointId;
	}

	/**
	 * @return the endPointId
	 */
	public String getEndpointId() {
		return endpointId;
	}

	/**
	 * @return the invoices
	 */
	public List<Invoice> getInvoices() {
		return invoices;
	}

	/**
	 * @param invoices the invoices to set
	 */
	public void setInvoices(List<Invoice> invoices) {
		this.invoices = invoices;
	}
	
}
