/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 *
 *
 */
package it.fe.core.par.utility;

import it.fe.core.par.common.PostalAddress;
import it.fe.core.par.invoice.AdditionalData;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;


/**
 * La classe <b><i>Meter</i></b> contiene le informazioni relative all'anagrafica contatori.
 * 
 * <b>N.B.</b> Sono presenti le annotazioni Hibernate/JPA per la persistenza sul database di FE.
 *
 */
@Entity
@Table(name="Meters")
public class Meter implements Serializable {
	
	private static final long serialVersionUID = 1L;
	
	public enum RegistrationUserType { AUTOMATICO }
	

	/**
	 * Numero progressivo assegnato al contatore
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private Long meterId;
	
	/**
	 * Codice che identifica il contatore: ad esempio la matricola
	 */
	private String meterNumber;
	
	/**
	 * Numero di presa: POD per l'elettricità, PDR per il gas
	 * in alcuni casi può essere nullo (es. TIA, acqua...)
	 */
	@ManyToOne( fetch=FetchType.EAGER, 
				cascade={CascadeType.PERSIST,CascadeType.MERGE} )
	@JoinColumn(name="utilityConsumptionPointId", referencedColumnName="utilityConsumptionPointId", nullable=true)
	private UtilityConsumptionPoint utilityConsumptionPoint;
	
	/**
	 * Indirizzo del contatore.
	 */
	@Embedded
	private PostalAddress meterAddress;

	/**
	 * Nome che identifica il contatore
	 */
	private String meterName;
	
	/**
	 * Coefficiente moltiplicativo (es. per l'energia elettrica k)
	 */
	private Double meterConstant;
	
	/**
	 * Data attivazione contatore
	 */
	@Temporal(TemporalType.DATE)
	private Date startDate;
	
	/**
	 * Data cessazione contatore
	 */
	@Temporal(TemporalType.DATE)
	private Date endDate;
	
	/**
	 * Settore merceologico
	 */
	@ManyToOne( fetch=FetchType.EAGER )
	@JoinColumn(name="sectorId", referencedColumnName="sectorId", nullable=false)
	private Sector sector;
	
	/**
	 * Informazioni fornitura
	 */
	@OneToMany( fetch=FetchType.EAGER, 
				cascade=CascadeType.ALL )
	@JoinColumn(name="meterId", referencedColumnName="meterId")
	private List<SupplyInfo> supplyInfoList;
	
	
	/**
	 * Data di registrazione nel db del contatore
	 */
	@Column(nullable=false)
	private Date registrationDate;
	
	/**
	 * Utente che ha effettuato la registrazione. 
	 * La registrazione può avvenire anche in automatico.
	 */
	@Column(nullable=false)
	private String registrationUser;
	
	/**
	 * Azione compiuta dall'utente. 
	 */
	@Column(nullable=false)
	private String action;

	/**
	 * Caratteristiche del contatore
	 */
	@OneToMany(mappedBy="meter")
	private List<MeterProperty> meterPropertyList;
		
	//--------------- Ctors --------------------------
	
	public Meter() {
		super();
	}

	public Meter(String meterNumber,
			PostalAddress meterAddress, Sector sector,
			Date registrationDate, String userIdentity, String action) {
		super();
		this.meterNumber = meterNumber;
		this.meterAddress = meterAddress;
		this.sector = sector;
		this.registrationDate = registrationDate;
		this.registrationUser = userIdentity;
		this.action = action;
	}

	
	//--------------- Getters/Setters --------------------------
	
	public Long getMeterId() {
		return meterId;
	}

	public void setMeterId(Long meterId) {
		this.meterId = meterId;
	}

	public String getMeterNumber() {
		return meterNumber;
	}

	public void setMeterNumber(String meterNumber) {
		this.meterNumber = meterNumber;
	}


	public void setUtilityConsumptionPoint(UtilityConsumptionPoint utilityConsumptionPoint) {
		this.utilityConsumptionPoint = utilityConsumptionPoint;
	}

	public UtilityConsumptionPoint getUtilityConsumptionPoint() {
		return utilityConsumptionPoint;
	}

	public PostalAddress getMeterAddress() {
		return meterAddress;
	}

	public void setMeterAddress(PostalAddress meterAddress) {
		this.meterAddress = meterAddress;
	}

	public void setMeterName(String meterName) {
		this.meterName = meterName;
	}

	public String getMeterName() {
		return meterName;
	}

	public void setMeterConstant(Double meterConstant) {
		this.meterConstant = meterConstant;
	}

	public Double getMeterConstant() {
		return meterConstant;
	}

	public Date getStartDate() {
		return startDate;
	}

	public void setStartDate(Date startDate) {
		this.startDate = startDate;
	}

	public Date getEndDate() {
		return endDate;
	}

	public void setEndDate(Date endDate) {
		this.endDate = endDate;
	}

	public Sector getSector() {
		return sector;
	}

	public void setSector(Sector sector) {
		this.sector = sector;
	}

	public List<SupplyInfo> getSupplyInfo() {
		return supplyInfoList;
	}

	public void setSupplyInfo(List<SupplyInfo> supplyInfo) {
		this.supplyInfoList = supplyInfo;
	}

	public Date getRegistrationDate() {
		return registrationDate;
	}

	public void setRegistrationDate(Date registrationDate) {
		this.registrationDate = registrationDate;
	}

	public String getRegistrationUser() {
		return registrationUser;
	}

	public void setRegistrationUser(String registrationUser) {
		this.registrationUser = registrationUser;
	}

	public String getAction() {
		return action;
	}

	public void setAction(String action) {
		this.action = action;
	}
	
	public void setAdditionalData(List<AdditionalData> additionalData) {
		this.additionalData = additionalData;
	}

	public List<AdditionalData> getAdditionalData() {
		return additionalData;
	}

	@ManyToMany(cascade={CascadeType.PERSIST,CascadeType.REFRESH,CascadeType.MERGE}, fetch=FetchType.LAZY)
	@JoinTable(name = "Meters_AdditionalData", joinColumns = @JoinColumn(name = "meterId"), inverseJoinColumns = @JoinColumn(name = "additionalDataId"))
	private List<AdditionalData> additionalData;

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((meterId == null) ? 0 : meterId.hashCode());
		result = prime * result
				+ ((meterNumber == null) ? 0 : meterNumber.hashCode());
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof Meter))
			return false;
		Meter other = (Meter) obj;
		if (meterId == null) {
			if (other.meterId != null)
				return false;
		} else if (!meterId.equals(other.meterId))
			return false;
		if (meterNumber == null) {
			if (other.meterNumber != null)
				return false;
		} else if (!meterNumber.equals(other.meterNumber))
			return false;
		return true;
	}

	/**
	 * @return the meterPropertyList
	 */
	public List<MeterProperty> getMeterPropertyList() {
		return meterPropertyList;
	}

	/**
	 * @param meterPropertyList the meterPropertyList to set
	 */
	public void setMeterPropertyList(List<MeterProperty> meterPropertyList) {
		this.meterPropertyList = meterPropertyList;
	}
	
	

}
