/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.process;

import it.fe.core.process.exception.JbpmProcessException;
import it.fe.core.process.proxy.ProcessEngineProxy;

import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.jbpm.api.Execution;
import org.jbpm.api.ProcessInstance;
import org.jbpm.api.task.Task;


/**
 * @author fulvio
 * 
*
 *
 * <p>Abstraction code on top of bpm processes</p>
 *
 */
public class ProcessService {
	
	private static Logger logger = Logger.getLogger(ProcessService.class); 
	
	public final static String FAILURE = "failure";
	public final static String SUCCESS = "success";
	public final static String NULL = "null";

	/**
	 * Starts a new process instance
	 * 
	 * @param processDefinitionKey - is the key of the process definition for which the latest version will be taken.
	 * @param processInstanceKey - is a user provided reference for the new execution that must be unique over all process versions with the same name
	 * @return the process instance id
	 */
	public static String start(String processDefinitionKey, String processInstanceKey) {
		if (processDefinitionKey == null || processDefinitionKey.trim().equals("")) {
			logger.error("Invalid process definition key "+ processDefinitionKey);
			throw new JbpmProcessException("Invalid process definition key "+ processDefinitionKey);
		}
		if (processInstanceKey == null || processInstanceKey.trim().equals("")) {
			logger.error("Invalid process instance key "+ processInstanceKey);
			throw new JbpmProcessException("Invalid process instance key "+ processInstanceKey);
		}
		// search for an already existent process instance
		if (ProcessService.findProcessInstanceById(processDefinitionKey, processInstanceKey) != null) {
			logger.error("Another process instance with the same identifier exists: " +
					createProcessInstanceId(processDefinitionKey, processInstanceKey));
			throw new JbpmProcessException("Another process instance with the same identifier exists: " +
					createProcessInstanceId(processDefinitionKey, processInstanceKey));
		}
		// create new process instance
		ProcessInstance processInstance = 
			ProcessEngineProxy
				.getExecutionService()
					.startProcessInstanceByKey(processDefinitionKey, processInstanceKey);
		
//		ProcessInstance processInstance = 
//			ProcessEngineProxy
//				.getExecutionService()
//					.startProcessInstanceById(processDefinitionKey, processInstanceKey);
		
		return processInstance.getId();
	}
	
	
	/**
	 * Starts a new process instance
	 * 
	 * @param processDefinitionKey - is the key of the process definition for which the latest version will be taken.
	 * @param variables - are the initial values of the process variables that will be set before the execution starts.
	 * @param processInstanceKey - is a user provided reference for the new execution that must be unique over all process versions with the same name
	 * @return the process instance id
	 */
	public static String start(String processDefinitionKey, Map<String, ?> variables, String processInstanceKey) {
		if (processDefinitionKey == null || processDefinitionKey.trim().equals("")) {
			logger.error("Invalid process instance identifier "+ processDefinitionKey);
			throw new JbpmProcessException("Invalid process instance identifier "+ processDefinitionKey);
		}
		if (variables == null || variables.isEmpty()) {
			logger.error("Provided variables map must not be null or empty");
			throw new JbpmProcessException("Provided variables map is null");
		}
		// search for an already existent process instance
		if (ProcessService.findProcessInstanceById(processDefinitionKey, processInstanceKey) != null) {
			logger.error("Another process instance with the same identifier exists: " +
					createProcessInstanceId(processDefinitionKey, processInstanceKey));
			throw new JbpmProcessException("Another process instance with the same identifier exists: " +
					createProcessInstanceId(processDefinitionKey, processInstanceKey));
		}
		// create new process instance
		ProcessInstance processInstance = 
			ProcessEngineProxy
				.getExecutionService()
					.startProcessInstanceByKey(processDefinitionKey, variables, processInstanceKey);
		
//		ProcessInstance processInstance = 
//			ProcessEngineProxy
//				.getExecutionService()
//					.startProcessInstanceById(processDefinitionKey, processInstanceKey);
		
		return processInstance.getId();
	}
	
    
    /**
     * Provides an external trigger to an execution.
     * 
     * @param processDefinitionKey
     * @param businessKey
     * @param currentState
     * @param username
     */
    public static String signal(String processDefinitionKey, String businessKey, String currentState, String username) {
        
    	if (processDefinitionKey == null || processDefinitionKey.trim().equals("")) {
        	logger.error("Invalid processDefinitionKey");
        	throw new JbpmProcessException("Invalid execution identifier");
        }
        if (businessKey == null || businessKey.trim().equals("")) {
        	logger.error("Invalid businessKey");
        	throw new JbpmProcessException("Invalid execution identifier");
        }
        if (currentState == null || currentState.trim().equals("")) {
        	logger.error("Invalid currentState");
        	throw new JbpmProcessException("Invalid execution identifier");
        }
        if (username == null || username.trim().equals("")) {
        	logger.error("Invalid username");
        	throw new JbpmProcessException("Invalid execution identifier");
        }
        
        ProcessInstance processInstance = findProcessInstanceById(processDefinitionKey, businessKey);
        
        if (processInstance == null) {
			throw new JbpmProcessException("Could not find process instance with id: "+processDefinitionKey+"."+businessKey);
		}
        
        Execution execution = processInstance.findActiveExecutionIn(currentState);
        
        if (execution == null) {
			throw new JbpmProcessException("Could not find active execution in state "+currentState+" for process instance with id: "+processDefinitionKey+"."+businessKey);
		}
        
        // signal execution
        processInstance = ProcessEngineProxy.getExecutionService().signalExecutionById(execution.getId());
        
        if (processInstance == null) {
        	logger.info("An error occurred while signalling process instance with id: "+processDefinitionKey+"."+businessKey+" from state "+currentState);
        	return ProcessService.FAILURE;
        }
        else {
        	logger.info("Process instance with id: "+processDefinitionKey+"."+businessKey+" successfully signalled from state "+currentState);
        	ProcessEngineProxy.getExecutionService().setVariable(processInstance.getId(), ProcessVariables.USERNAME, username);
        	return ProcessService.SUCCESS;
        }
        
    }
    
   
    /**
     * Provides an external trigger to an execution.
     * 
     * @param processDefinitionKey
     * @param businessKey
     * @param signalName
     * @param currentState
     * @param username
     */
    public static String signal(String processDefinitionKey, String businessKey, String signalName, String currentState, String username) {
    	if (processDefinitionKey == null || processDefinitionKey.trim().equals("")) {
        	logger.error("Invalid processDefinitionKey");
        	throw new JbpmProcessException("Invalid execution identifier");
        }
        if (businessKey == null || businessKey.trim().equals("")) {
        	logger.error("Invalid businessKey");
        	throw new JbpmProcessException("Invalid execution identifier");
        }
        if (signalName == null || signalName.trim().equals("")) {
        	logger.error("Invalid signalName");
        	throw new JbpmProcessException("Invalid execution identifier");
        }
        if (currentState == null || currentState.trim().equals("")) {
        	logger.error("Invalid currentState");
        	throw new JbpmProcessException("Invalid execution identifier");
        }
        if (username == null || username.trim().equals("")) {
        	logger.error("Invalid username");
        	throw new JbpmProcessException("Invalid execution identifier");
        }
        
        ProcessInstance processInstance = findProcessInstanceById(processDefinitionKey, businessKey);
        
        if (processInstance == null) {
			throw new JbpmProcessException("Could not find process instance with id: "+processDefinitionKey+"."+businessKey);
		}
        
        Execution execution = processInstance.findActiveExecutionIn(currentState);
        
        if (execution == null) {
			throw new JbpmProcessException("Could not find active execution in state "+currentState+" for process instance with id: "+processDefinitionKey+"."+businessKey);
		}
        
        // signal execution
        ProcessEngineProxy.getExecutionService().setVariable(processInstance.getId(), ProcessVariables.USERNAME, username);
        processInstance = ProcessEngineProxy.getExecutionService().signalExecutionById(execution.getId(), signalName);
        
        if (processInstance == null) {
        	logger.info("An error occurred while signalling process instance with id: "+processDefinitionKey+"."+businessKey+" from state "+currentState+" with signal "+signalName);
        	return ProcessService.FAILURE;
        }
        else {
        	logger.info("Process instance with id: "+processDefinitionKey+"."+businessKey+" successfully signalled from state "+currentState+" with signal "+signalName);
        	return ProcessService.SUCCESS;
        }
    }
    
    /**
     * List of tasks that are assigned to the given user. Returns an empty list in case no such tasks exist.
     * 
     * @param userId
     * @return
     */
    public static List<Task> findPersonalTasks(String userId) {
    	if (userId == null || userId.trim().equals("")) {
    		logger.error("Invalid user identifier");
    		throw new JbpmProcessException("Invalid execution identifier");
    	}
    	return ProcessEngineProxy.getTaskService().findPersonalTasks(userId);
    }
    
    /**
     * Deletes this task, marks the related history task as completed. 
     * If the task was created in the context of a process execution, this operation may result 
     * in a process instance being triggered.
     * 
     * @param taskId
     */
    public static void completeTask(String taskId) {
    	if (taskId == null || taskId.trim().equals("")) {
        	logger.error("Invalid task identifier");
        	throw new JbpmProcessException("Invalid task identifier");
        }
    	ProcessEngineProxy.getTaskService().completeTask(taskId);
    }
    
    /**
     * Deletes this task, marks the related history task as completed with the specified outcome. 
     * If the task was created in the context of a process execution, this operation may result in 
     * a process instance being triggered. The outcome in that case corresponds to an outgoing 
     * transition in the process.
     * 
     * @param taskId
     * @param outcome
     */
    public static void completeTask(String taskId, String outcome) {
    	if (taskId == null || taskId.trim().equals("")) {
        	logger.error("Invalid task identifier");
        	throw new JbpmProcessException("Invalid task identifier");
        }
    	ProcessEngineProxy.getTaskService().completeTask(taskId, outcome);
    }
  
    /**
     * Ends a process instance 
     * 
     * @param processDefinitionKey
     * @param processInstanceKey
     */
    public static void end(String processDefinitionKey, String processInstanceKey) {
    	if (processDefinitionKey == null || processDefinitionKey.trim().equals("")) {
    		logger.error("Invalid process definition key: "+processDefinitionKey);
    		throw new JbpmProcessException("Invalid process definition key");
    	}
    	if (processInstanceKey == null || processInstanceKey.trim().equals("")) {
    		logger.error("Invalid process instance key: "+processInstanceKey);
    		throw new JbpmProcessException("Invalid process instance key");
    	}
        // search for an already existent process instance
        ProcessInstance processInstance = ProcessService.findProcessInstanceById(processDefinitionKey, processInstanceKey);
        if (processInstance == null) {
        	logger.error("No process instance found for identifier: " +
        			createProcessInstanceId(processDefinitionKey, processInstanceKey));
        	throw new JbpmProcessException("No process instance found for identifier: " +
        			createProcessInstanceId(processDefinitionKey, processInstanceKey));
        }
        
        ProcessEngineProxy
        	.getExecutionService()
        		.endProcessInstance(processInstance.getId(), Execution.STATE_ENDED);
    }
    
	/**
	 * 
	 * search for an execution that is active and in the given activityName
	 * 
	 * @param processDefinitionKey
	 * @param businessKey
	 * @param activityName
	 * @return
	 */
	public static String findActiveExecutionIn(String processDefinitionKey, String businessKey, String activityName) {
		ProcessInstance processInstance = findProcessInstanceById(processDefinitionKey, businessKey);
		if (processInstance != null) {
			Execution execution = processInstance.findActiveExecutionIn(activityName);
			if (execution == null) {
				return ProcessService.FAILURE;
			}
			else {
				return ProcessService.SUCCESS;
			}
		}
		else
			return ProcessService.FAILURE;
		
	}
	
	/**
	 * Returns a process instance
	 * 
	 * @param processDefinitionKey
	 * @param processInstanceKey
	 * @return 
	 */
	public static ProcessInstance findProcessInstanceById(String processDefinitionKey, String processInstanceKey) {
		return 
			ProcessEngineProxy
				.getExecutionService()
					.findProcessInstanceById(
							ProcessService.createProcessInstanceId(processDefinitionKey, processInstanceKey));
	}

	
	// ------------------------------------------------------------------------------
	// UTILS
	// ------------------------------------------------------------------------------
	
	private static String createProcessInstanceId(String processDefinitionKey, String processInstanceKey) {
		return processDefinitionKey+"."+processInstanceKey;
	}
	
}
