/**
 * This file is part of FatturazioneElettronica - ESB
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - ESB is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - ESB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - ESB.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.esb.storage;

import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.par.common.FEFile;
import it.fe.core.par.invoice.OriginalDocument;
import it.fe.core.par.invoice.TransmissionType;
import it.fe.esb.common.CheckedDocument;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.activation.MimetypesFileTypeMap;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Part;
import javax.mail.internet.InternetAddress;

import org.apache.log4j.Logger;



public class EMailStorage {
	
	protected static Logger logger = Logger.getLogger(EMailStorage.class.getName());
	
	private static boolean textIsHtml = false;
	
	public CheckedDocument readData(Message message) {
		CheckedDocument out = new CheckedDocument();
		
		List<FEFile> attachements = new ArrayList<FEFile>();
		OriginalDocument originalDocument = new OriginalDocument();
		
		try {
			//Data ricezione
			Date emailReceiptDate = message.getSentDate();
			originalDocument.setReceiptDateTime(emailReceiptDate);
			logger.info("Data: " + emailReceiptDate);
			
			//E-mail mittente
			InternetAddress fromAddress = (InternetAddress)message.getFrom()[0];
			String emailSender = fromAddress.toString();
			logger.info("Mittente: " + emailSender);
			originalDocument.setSender(emailSender);
			
	        String to = InternetAddress.toString(
	                message.getRecipients(Message.RecipientType.TO));
	        if (to != null) {
	        	logger.info("To: " + to);
	        }
			
			//Oggetto
			String emailSubject = message.getSubject();
			logger.info("Oggetto: " + emailSubject);
			
			//Testo e allegati
			String emailText = "";
			byte[] emailAttachment = null;
			String emailAttachmentName = "";
			
			InputStream is;

			if(!(message.isMimeType("multipart/*"))){
				logger.error("E-MAIL SENZA ALLEGATO");
				
				emailText = (String)message.getContent();
				logger.info("Body: " + emailText);
				originalDocument.setAdditionalInfo(emailText);
			}
			else {
				Multipart multipart = (Multipart) message.getContent();
				Part part;
				
				for (int i=0, n=multipart.getCount(); i<n; i++) {
					part = multipart.getBodyPart(i);
	
					emailAttachmentName = part.getFileName();
					
					//se la parte non ha FileName allora è testuale
					if(emailAttachmentName == null) {
						emailText = getText(part);
						logger.info("Body: " + emailText);
						originalDocument.setAdditionalInfo(emailText);
					}
					else {
						is = part.getInputStream();
						emailAttachment = inputStreamToBytes(is);
						logger.info("Allegato: " + emailAttachmentName);
						
						if (emailAttachment != null) {
							FEFile document = new FEFile(emailAttachment, emailAttachmentName);
							document.setFileMimeType(new MimetypesFileTypeMap().getContentType(emailAttachmentName));
							
							attachements.add(document);
						}
					}				
				}
			}

			originalDocument.setReceiptDocuments(attachements);
			
			originalDocument.setTransmissionType(
				JNDIUtils.getTransmissionTypeManager().getTransmissionTypeByDescription(TransmissionType.type.EMAIL.toString())
			);
						
			long originalDocumentId = saveIntoDb(originalDocument);
			originalDocument.setOriginalDocumentId(originalDocumentId);
			
			out.setOriginalDocument(originalDocument);
			
			
			if (originalDocument.getReceiptDocuments() == null) {
				JNDIUtils.getOriginalDocumentManager().updateReceiveStatus(
						originalDocumentId, 
						OriginalDocument.receiveStatusType.ERRORE.toString(),
						"E-Mail senza allegati");
			}
			else {
				out.setDocumentList(attachements);
			}
				
		} catch (MessagingException e) {
			logger.error(e.getMessage());
		} catch (IOException e) {
			logger.error(e.getMessage());
		}
	
		return out;
	}
	
	public long saveIntoDb(OriginalDocument originalDocument) {
				
		long originalDocumentId = JNDIUtils.getOriginalDocumentManager().insertOriginalDocument(originalDocument);
		logger.info("Salvata e-mail nel db");
		
		return originalDocumentId;
	}
	
	
	
	/**
	 * Ritorna il testo primario contente il messaggio.
	 * Il controllo dà la precedenza all'estrazione di testo semplice, non html.
	 * 
	 * @param
	 *     (Part) parte del bodypart da "pulire"
	 * 
	 * @return
	 * 		(String) il messaggio contentuo nel bodypart della e-mail
	 *  
	 *  
	 * @link Ref. to JavaMail FAQ:  http://java.sun.com/products/javamail/FAQ.html#mainbody
	 */
    private static String getText(Part p) throws MessagingException, IOException {
        if (p.isMimeType("text/*")) {
            String s = (String)p.getContent();
            textIsHtml = p.isMimeType("text/html");
            return s;
        }

        if (p.isMimeType("multipart/alternative")) {
            // prefer plain text over html text
            Multipart mp = (Multipart)p.getContent();
            String text = null;
            for (int i = 0; i < mp.getCount(); i++) {
                Part bp = mp.getBodyPart(i);
                if (bp.isMimeType("text/html")) {
                    if (text == null)
                        text = getText(bp);
                    
                    continue;
                } else if (bp.isMimeType("text/plain")) {
                    String s = getText(bp);
                    if (s != null)
                        return s;
                } else {
                    return getText(bp);
                }
            }
            return text;
        } else if (p.isMimeType("multipart/*")) {
            Multipart mp = (Multipart)p.getContent();
            for (int i = 0; i < mp.getCount(); i++) {
                String s = getText(mp.getBodyPart(i));
                if (s != null)
                    return s;
            }
        }

        return null;
    }	

	
	public byte[] inputStreamToBytes(InputStream in)  throws IOException {

		ByteArrayOutputStream out = new ByteArrayOutputStream(1024);
		byte[] buffer = new byte[1024];
		int len;
		   
		while((len = in.read(buffer)) >= 0)
			out.write(buffer, 0, len);

		in.close();
		out.close();
		   
		return out.toByteArray();
	}
	
}
