/**
 * This file is part of FatturazioneElettronica - Signature and timestamp facilities
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Signature and timestamp facilities is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Signature and timestamp facilities is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Signature and timestamp facilities.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.trento.comune.j4sign.verification;

import java.io.*;
import java.security.*;
import java.security.cert.*;
import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.logging.Logger;

import javax.security.auth.x500.X500Principal;

import it.trento.comune.j4sign.verification.servlet.Verifier;

import org.bouncycastle.asn1.pkcs.*;
import org.bouncycastle.cms.*;
import org.bouncycastle.util.encoders.Base64;


/**
 * Object used to perform verification about certificate validity and signature
 * integrity. Methods get... perform action, methods is... just return value. It
 * is obviously necessary performing verification before returning the value <br>
 * <br>
 * Oggetto che effettua e restituisce le verifiche sul certificato e
 * sull'integrità della firma. I metodi get perfomano la verifica, i metodi is
 * restituiscono solo il risultato. E' ovviamente necessario prima effettuare la
 * verifica e poi restituire il risultato.
 * 
 * @author Francesco Cendron
 */
public class VerifyResult {

	private Logger log = Logger.getLogger(this.getClass().getName());
	private boolean isPathValid;

	private boolean isSignatureCorrect;

	private boolean isRevoked;
	private boolean isExpired;
	private boolean isInUse; // contrario di NotYetValid

	private CertValidity cv;

	private String CRLerror = "";
	private String certPathError = "";

	private SignerInformation signer;
	private CMSSignedData cms;
	private X509Certificate cert;
	private String encodedDigest;
	private boolean isDownloadCRLForced;

	private boolean isPassed;

	private Hashtable<X500Principal, VerifyResult> risultatiCs;

	/**
	 * Constructor
	 * 
	 * @param c
	 *            X509Certificate
	 * @param cm
	 *            CMSSignedData
	 * @param C
	 *            CertificationAuthorities
	 * @param s
	 *            SignerInformation
	 */
	public VerifyResult(CertStore certs, X509Certificate c, CMSSignedData cm,
			CertificationAuthorities C, SignerInformation s) {
		this(certs, c, cm, C, s, false);

	}

	/**
	 * Constructor
	 * 
	 * @param c
	 *            X509Certificate
	 * @param cm
	 *            CMSSignedData
	 * @param C
	 *            CertificationAuthorities
	 * @param s
	 *            SignerInformation
	 * @param isDownloadCRLForced
	 *            boolean
	 */
	public VerifyResult(CertStore certs, X509Certificate c, CMSSignedData cm,
			CertificationAuthorities roots, SignerInformation s,
			boolean isDownloadCRLForced) {

		isSignatureCorrect = false;
		cert = c;
		signer = s;
		this.isDownloadCRLForced = isDownloadCRLForced;
		cv = new CertValidity(c, roots, this.isDownloadCRLForced);

		cms = cm;
		encodedDigest = null;
		isPassed = false;

		initCountersignatures(certs, cm, roots, s, isDownloadCRLForced);

	}

	private void initCountersignatures(CertStore certs, CMSSignedData cm,
			CertificationAuthorities roots, SignerInformation parentSigner,
			boolean isDownloadCRLForced) {

		SignerInformationStore cs = parentSigner.getCounterSignatures();
		if (cs.size() > 0) {

			risultatiCs = new Hashtable();

			log.info("detected " + cs.size() + " countersignatures for "
					+ cert.getSubjectX500Principal());

			Iterator csIterator = cs.getSigners().iterator();

			while (csIterator.hasNext()) {
				SignerInformation counterSigner = (SignerInformation) csIterator
						.next();
				Collection cc = null;
				try {
					cc = certs.getCertificates(counterSigner.getSID());

				} catch (CertStoreException ex1) {
					log.severe("Errore nel CertStore");
					// errors.add("Errore nel CertStore");
				}

				if (cc.size() == 1) {

					X509Certificate c = (X509Certificate) cc.toArray()[0];

					log.info("CounterSigner CN: " + Verifier.getCommonName(c));

					VerifyResult vr = new VerifyResult(certs, c, cm, roots,
							counterSigner, isDownloadCRLForced);

					risultatiCs.put(c.getSubjectX500Principal(), vr);

				}

			}
		}
	}

	public X509CRL getCrl() {
		return cv.getCRL();
	}

	public X509Certificate getCert() {
		return cert;
	}

	/**
	 * Verify that public Key corresponds<br>
	 * <br>
	 * Effettua la verifica che la chiave pubblica usata corrisponda
	 * 
	 * @return boolean
	 */
	public boolean getSignatureCorrect() {
		try {

			isSignatureCorrect = signer.verify(cert, "BC");

		} catch (CMSException ex) {
			log.severe(ex.getMessage());
		} catch (CertificateNotYetValidException ex) {
			log.severe(ex.getMessage());
		} catch (CertificateExpiredException ex) {
			log.severe(ex.getMessage());
		} catch (NoSuchProviderException ex) {
			log.severe(ex.getMessage());
		} catch (NoSuchAlgorithmException ex) {
			log.severe(ex.getMessage());
		}
		log.info(signer.getSID().toString() + " " + cert.getSubjectDN() + " "
				+ isSignatureCorrect);
		return isSignatureCorrect;
	}

	public boolean isSignatureCorrect() {
		return isSignatureCorrect;
	}

	/**
	 * Perform the global verification and return the global result<br>
	 * <br>
	 * Metodo complessivo che esegue e restituisce la verifica
	 * 
	 * @return boolean
	 */
	public boolean getPassed() {

		boolean isCorrect = getSignatureCorrect();
		boolean isCertValid = cv.getPassed();

		isPassed = isCorrect && isCertValid;
		
		if (risultatiCs != null) {

			Iterator<X500Principal> principalIterator = risultatiCs.keySet()
					.iterator();

			boolean csVerified = false;
			int i= 0;
			while (principalIterator.hasNext()) {
				X500Principal signer = principalIterator.next();
				VerifyResult sv = risultatiCs.get(signer);
				csVerified = sv.getPassed() && ((i == 0) || csVerified);
			}
			
			isPassed = isPassed && csVerified;

		}
		 
		CRLerror = cv.getCRLerror();
		certPathError = cv.getCertPathError();

		return isPassed;
	}

	public Hashtable<X500Principal, VerifyResult> getRisultatiCs() {
		return risultatiCs;
	}

	/**
	 * Perform the global verification and return the global result<br>
	 * <br>
	 * Metodo complessivo che esegue e restituisce la verifica
	 * 
	 * @return boolean
	 */
	public boolean getPassed_cnipasigner_expired() {

		isPassed = getSignatureCorrect() && cv.getPassed_noExpiredCheck();
		CRLerror = cv.getCRLerror();
		return isPassed;
	}

	/**
	 * Return CRLerror (error during CRL download)<br>
	 * <br>
	 * Restituisce l'errore CRLerror (errore durante il download della CRL)
	 * 
	 * @return String
	 */
	public String getCRLerror() {

		return CRLerror;
	}

	public String getCertPathError() {
		return certPathError;
	}

	/**
	 * Checks certification path by IssuerX500Principal keyed in CAroot<br>
	 * <br>
	 * risale il certification path attraverso IssuerX500Principal chiave in
	 * CAroot
	 * 
	 * @return boolean
	 */
	public boolean isPathValid() {

		return cv.isPathValid();
	}

	public boolean isRevoked() {

		return cv.isRevoked();
	}

	public boolean isExpired() {

		return cv.getExpired();
	}

	public boolean isInUse() {

		return cv.getInUse();
	}

	public boolean isPassed() {

		return isPassed;
	}

	public boolean isDownloadCRLForced() {

		return isDownloadCRLForced;
	}

	/**
	 * Checks if CRL is already been checked<br>
	 * <br>
	 * True se la CRL è stata verificata
	 * 
	 * @return boolean
	 */
	public boolean isCRLChecked() {

		return cv.isCRLChecked();
	}

	public void setPassed(boolean b) {
		isPassed = b;
	}

	/**
	 * Creates the base64 encoding of a byte array.
	 * 
	 * @param bytes
	 *            byte[]
	 * @return java.lang.String
	 */
	public String encodeFromBytes(byte[] bytes) {


		 String encString = new String(Base64.encode(bytes));

		return encString;
	}

	/**
	 * Return signed content
	 * Ritorna il contenuto firmato (la firma)
	 * 
	 * @return byte[]
	 */
	byte[] getRawBytes() {
		return (byte[]) cms.getSignedContent().getContent();
	}

}
