/**
 * This file is part of FatturazioneElettronica - Signature and timestamp facilities
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Signature and timestamp facilities is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Signature and timestamp facilities is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Signature and timestamp facilities.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.trento.comune.j4sign.verification.servlet;


import it.trento.comune.j4sign.verification.RootsVerifier;
import it.trento.comune.j4sign.verification.VerifyResult;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.security.MessageDigest;
import java.security.cert.X509Certificate;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.Enumeration;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.util.logging.Logger;

import javax.security.auth.x500.X500Principal;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.swing.JFrame;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItem;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.FileCleanerCleanup;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.bouncycastle.asn1.ocsp.Request;
import org.bouncycastle.cms.CMSException;
import org.bouncycastle.cms.CMSSignedData;

/**
 * Servlet implementation class for Servlet: ServletVerifica
 * 
 */
public class VerifyServlet extends javax.servlet.http.HttpServlet implements
		javax.servlet.Servlet {

	private Logger log = Logger.getLogger(this.getClass().getName());

	private final static long MAX_UPLOAD_SIZE = 10000L;
	private final static long UPLOADED_DIR_SIZE_FACTOR = 2;

	private long maxUploadSize = MAX_UPLOAD_SIZE;
	private long maxUploadDirSize = MAX_UPLOAD_SIZE * UPLOADED_DIR_SIZE_FACTOR;

	private String fingerprintString = null;
	private String fingerprintGU = null;

	ServletFileUpload upload;

	@Override
	public void init() throws ServletException {

		super.init();

		try {
			String mus = this.getInitParameter("maxUploadSize");

			if (mus != null) {
				maxUploadSize = Long.valueOf(mus).longValue();
				log
						.fine("MaxUploadSize set from servlet \"maxUploadSize\" init parameter");
			} else
				log
						.info("\"maxUploadSize\" servlet init parameter not present");
		} catch (NumberFormatException nfe) {
			log.severe(nfe.getMessage());
		}

		try {
			String mus = this.getInitParameter("uploadedDirSizeFactor");

			if (mus != null) {
				maxUploadDirSize = Long.valueOf(mus).longValue()
						* maxUploadSize;
				log
						.fine("uploadedDirSizeFactor set from servlet \"maxUploadSize\" init parameter");
			} else
				log
						.info("\"uploadedDirSizeFactor\" servlet init parameter not present");
		} catch (NumberFormatException nfe) {
			log.severe(nfe.getMessage());
		}

		// Create a factory for disk-based file items
		DiskFileItemFactory factory = new DiskFileItemFactory();

		// Create a new file upload handler
		upload = new ServletFileUpload(factory);

		// maximum accepted size, upload is stopped (during
		// upload.parseRequest()) if exceeded

		upload.setFileSizeMax(maxUploadSize);
		log.fine("MaxUploadSize set to: " + maxUploadSize + " bytes");

		String afingerprintString = this.getServletContext().getInitParameter(
				"rootsSignatureCertFingerprint");
		if (afingerprintString != null) {
			try {
				decodeHex(afingerprintString.toCharArray());
				log.info("\"rootsSignatureCertFingerprint\"="
						+ afingerprintString + " decoded");
				fingerprintString = afingerprintString;
			} catch (Exception e) {
				log.severe(e.getMessage());
				log.severe("\"rootsSignatureCertFingerprint\"="
						+ afingerprintString + " not decoded");
			}

		} else
			log
					.severe("\"rootsSignatureCertFingerprint\" context init parameter not present, cannot perform validarion!");

		fingerprintGU = this.getServletContext().getInitParameter(
				"fingerprintGU");
		if (fingerprintGU == null)
			log
					.warning("\"fingerprintGU\" = context init parameter not present");
	}

	/*
	 * (non-Java-doc)
	 * 
	 * @see javax.servlet.http.HttpServlet#doGet(HttpServletRequest request,
	 * HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {

		log.info("GET in corso...");

		HttpSession session = request.getSession(false);

		String fileName = request.getPathInfo().substring(1);
		String errorMsg = null;

		if (session == null) {
			log.warning("session == null!");
			sendErrorMsg(response, "Sessione scaduta o cookies disabilitati.");
			return;
		}

		if (fileName == null) {
			log.warning("fileName == null! - error getting pathInfo");
			sendErrorMsg(response, "Errore nel recupero del path info.");
			return;
		}

		String subDir = session.getId();

		String realFileName = (String) session.getAttribute(fileName + ".p7m");

		String fs = System.getProperty("file.separator");
		File f = new File(getServletContext().getRealPath("/WEB-INF/files")
				+ fs + subDir + fs + realFileName);

		if (f.exists()) {
			// Setting correct content type
			response.setContentType(getServletContext().getMimeType(fileName));
			ServletOutputStream os = response.getOutputStream();

			try {
				byte[] contentBytes = (byte[]) new CMSSignedData(Verifier
						.getSignedContentBytes(f.getAbsolutePath()))
						.getSignedContent().getContent();

				sendContent(contentBytes, os);
				os.flush();

				log.info("Sent \"" + realFileName + "\" (" + fileName
						+ ") to client " + request.getRemoteAddr()
						+ " in session " + subDir);

			} catch (CMSException e) {
				log.severe(e.getMessage());
				errorMsg = "Errore nell'estrazione del contenuto: "
						+ e.getMessage();
			}
		} else{
			errorMsg = "Il file non è stato trovato";
			log.warning("file '"+f.getPath()+"' not found!");
		}
		if (errorMsg != null) 
			sendErrorMsg(response, errorMsg);
	}

	private void sendErrorMsg(HttpServletResponse response, String errorMsg) {
		response.setContentType("text/html");
		try {
			response.getWriter().println("<h3>Errore</h3><p>" + errorMsg + "</p>");
		} catch (IOException e) {
			log.severe(e.getMessage());
		}

	}

	/*
	 * (non-Java-doc)
	 * 
	 * @see javax.servlet.http.HttpServlet#doPost(HttpServletRequest request,
	 * HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {

		StringBuffer header = new StringBuffer("");
		Vector<String> errors = new Vector<String>();
		Vector<ProcessedItem> processedItems = new Vector<ProcessedItem>();

		response.setContentType("text/html");

		log.info("POST in corso...");

		HttpSession session = request.getSession(false);

		// We set maximum session duration of a session after second request
		// equal to MaxInactiveInterval, so that the possibility to upload new
		// files
		// is at maximum 2 * MaxInactiveInterval long.
		// When session is invalidated, all related uploaded file are cleaned by
		// the session listener.

		long duration = -1;
		if (session != null) {

			duration = session.getLastAccessedTime()
					- session.getCreationTime();

			if (duration > (session.getMaxInactiveInterval() * 1000)) {
				session.invalidate();
				duration = 0;
				session = request.getSession(true);

			}
		} else
			session = request.getSession(true);

		if (session != null)
			if (fingerprintString == null)
				errors
						.add("Impronta SHA1 del file delle Autorità di Certificazione non configurata; impossibile eseguire la verifica");
			else {
				// Check that we have a file upload request
				boolean isMultipart = ServletFileUpload
						.isMultipartContent(request);

				if (isMultipart) {
					String sessionId = session.getId();

					header
							.append("Sessione "
									+ sessionId
									+ ((duration > 0) ? (": " + duration / 1000 + " secondi")
											: ""));

					// Parse the request
					try {
						List<DiskFileItem> items = upload.parseRequest(request);

						if (items.isEmpty()) {
							errors.add("Nessun elemento da elaborare!!");
							log.warning("No element to process");
						}
						// Process the uploaded items
						Iterator<DiskFileItem> iter = items.iterator();
						while (iter.hasNext()) {
							ProcessedItem pi = null;
							DiskFileItem item = iter.next();

							if (item.isFormField()) {
								pi = processFormField(item);

							} else {

								pi = processUploadedFile(item, sessionId);
								if (pi != null)
									session.setAttribute(pi.getFileName(), pi
											.getWritedFileName());

								log.info("Processed uploaded file \""
										+ pi.getFileName() + "\" from client "
										+ request.getRemoteAddr()
										+ " in session " + sessionId);
							}

							if (pi != null)
								processedItems.add(pi);

						}

					} catch (FileUploadException e) {
						if (e instanceof org.apache.commons.fileupload.FileUploadBase.FileSizeLimitExceededException) {
							log.severe(e.getMessage());
							errors.add("Il file supera il limite di "
									+ upload.getFileSizeMax() + " bytes");
						} else {
							log.severe(e.getMessage());
							errors.add(e.getMessage());
						}
					}
				}

				request.setAttribute("processedItems", processedItems);
			}

		request.setAttribute("headerVerifica", header);
		request.setAttribute("errors", errors);
		this.getServletContext().getRequestDispatcher("/verifica.jsp").forward(
				request, response);

	}

	private ProcessedItem processUploadedFile(DiskFileItem item, String subDir) {
		String baseFileName = null;
		ProcessedItem pi = null;
		// Process a file upload
		if (!item.isFormField()) {

			pi = new ProcessedItem(item.getFieldName(), !item.isFormField());
			String fileName = item.getName();

			// IE insists to provide full path, get only file name (via commons
			// IO).
			if (fileName != null) {
				baseFileName = FilenameUtils.getName(fileName);
				pi.setFileName(baseFileName);
				pi.setSize(item.getSize());
				pi.setInMemory(item.isInMemory());
				pi.setContentType(item.getContentType());
			}

			if (baseFileName != null)
				if (!baseFileName.endsWith(".p7m"))
					pi.getErrors().add(
							"Il nome del file non termina con \".p7m\"!");
				else {

					File filesDir = new File(getServletContext().getRealPath(
							"/WEB-INF/files"));

					File sd = new File(filesDir, subDir);

					sd.mkdirs();

					// check maximum size for directory containing uploaded
					// files

					if (FileUtils.sizeOfDirectory(filesDir) > maxUploadDirSize) {
						log
								.warning("MaxUploadDirSize of "
										+ maxUploadDirSize
										+ " bytes exceeded, not accepting more files at the moment");
						pi
								.getErrors()
								.add(
										"Impossibile verificare ulteriori file, attendere qualche minuto!");

					} else {

						String writedFileName;
						try {
							writedFileName = getMD5Checksum(item
									.getInputStream());

							File uploadedFile = new File(sd, writedFileName);
							// File uploadedFile = item.getStoreLocation();

							item.write(uploadedFile);

							pi.setWritedFileName(writedFileName);
							// pi.setWritedFileName(uploadedFile.getName());

							pi.setFullFileName(uploadedFile.getAbsolutePath());

						} catch (Exception e) {
							log.severe(e.getMessage());
							pi.getErrors().add(e.getMessage());
						}

						if (pi.getErrors().isEmpty()) {
							String confPath = getServletContext().getRealPath(
									"/WEB-INF")
									+ System.getProperty("file.separator")
									+ "conf";

							// Inizio verifica
							RootsVerifier rv;
							Verifier v = null;
							try {
								rv = RootsVerifier.getInstance(confPath,
										decodeHex(this.fingerprintString
												.toCharArray()));

								v = new Verifier(pi.getFullFileName(), false,
										rv);

							} catch (Exception e) {
								pi.getErrors().add(
										"Errore nell'inizializzazione della verifica: "
												+ e.getMessage());
							}

							if (v != null) {
								while (!v.isDone())
									v.verify();

								if (!v.getErrors().isEmpty())
									pi.getErrors().addAll(v.getErrors());

								Hashtable<X500Principal, VerifyResult> r = v
										.getRisultati();

								if (r != null) {
									Iterator<X500Principal> principalIterator = r
											.keySet().iterator();

									boolean isVerified = false;
									int i = 0;
									while (principalIterator.hasNext()) {
										X500Principal signer = principalIterator
												.next();
										VerifyResult sv = r.get(signer);
										isVerified = sv.isPassed()
												&& ((i == 0) || isVerified);

									}

									pi.setVerifyResults(r);
									pi.setVerified(isVerified);

								}
							}
						}

						String o1 = pi.getFileName();
						pi.setContentFileName(o1.substring(0, o1
								.lastIndexOf(".")));

					}
				}

		}
		return pi;
	}

	private ProcessedItem processFormField(DiskFileItem item) {
		ProcessedItem pi = null;
		// Process a regular form field
		if (item.isFormField()) {
			pi = new ProcessedItem(item.getFieldName(), !item.isFormField());
			pi.setString(item.getString());
		}

		return pi;
	}

	private void sendContent(byte[] bytes, OutputStream os) throws IOException {

		ByteArrayInputStream bais = new ByteArrayInputStream(bytes);

		byte[] buffer = new byte[4096]; // A buffer to hold file contents
		int bytes_read; // How many bytes in buffer
		// Read a chunk of bytes into the buffer, then write them out,
		// looping until we reach the end of the file (when read() returns -1).
		// Note the combination of assignment and comparison in this while
		// loop. This is a common I/O programming idiom.
		while ((bytes_read = bais.read(buffer)) != -1) { // Read bytes until EOF
			os.write(buffer, 0, bytes_read); // write bytes
		}

	}

	private static byte[] createChecksum(InputStream fis) throws Exception {

		byte[] buffer = new byte[1024];
		MessageDigest complete = MessageDigest.getInstance("MD5");
		int numRead;
		do {
			numRead = fis.read(buffer);
			if (numRead > 0) {
				complete.update(buffer, 0, numRead);
			}
		} while (numRead != -1);
		fis.close();
		return complete.digest();
	}

	public static String getMD5Checksum(InputStream fis) throws Exception {
		byte[] b = createChecksum(fis);
		String result = "";
		for (int i = 0; i < b.length; i++) {
			result += Integer.toString((b[i] & 0xff) + 0x100, 16).substring(1);
		}
		return result;
	}

	public static byte[] decodeHex(char[] data) throws Exception {

		int len = data.length;

		if ((len & 0x01) != 0) {
			throw new Exception("Odd number of characters.");
		}

		byte[] out = new byte[len >> 1];

		// two characters form the hex value.
		for (int i = 0, j = 0; j < len; i++) {
			int f = toDigit(data[j], j) << 4;
			j++;
			f = f | toDigit(data[j], j);
			j++;
			out[i] = (byte) (f & 0xFF);
		}

		return out;
	}

	protected static int toDigit(char ch, int index) throws Exception {
		int digit = Character.digit(ch, 16);
		if (digit == -1) {
			throw new Exception("Illegal hexadecimal character " + ch
					+ " at index " + index);
		}
		return digit;
	}

}