/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.statistics;

import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.ejb.generic.NestedCriterion;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.invoice.InvoiceState;
import it.fe.core.par.user.ProfileDetail;
import it.fe.core.par.user.Role;
import it.fe.web.common.DateUtils;
import it.fe.web.common.SessionUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Restrictions;

/**
 * Controller web per le statistiche
 * 
 * 
 * @author stefano
 *
 */
public class StatisticsController {

	/**
	 * Numero di fatture presenti nel sistema
	 */
	private Long invoiceNumber =  new Long(-1);
	
	/**
	 * Data inizio operatività F.E.: la assumiamouguale alla data della prima ricezione
	 */
	private Date firstDocument;
	
	/**
	 * Numero di fatture in ciascuno stato attivo
	 */
	private List<InvoiceStatisticsHolder> invoiceStates = null;

	
	/**
	 * Numero di fatture in ciascuno stato terminato
	 */
	private List<InvoiceStatisticsHolder> endedInvoiceStates = null;
	
	/**
	 * Numero di fatture rispettivamente a - meno di una settimana dalla
	 * scadenza - a 7-14 giorni dalla scadenza - a più di 14 giorni dalla
	 * scadenza
	 */
	private List<InvoiceStatisticsHolder> expiringInvoices = null;

	/**
	 * Tempi medi e massimi di elaborazione di fatture per stato
	 */
	private List<TimingHolder> invoiceHandlingTimesPerState = null;
	
	/**
	 * Tempi medi e massimi di elaborazione di fatture per stato e utente
	 */
	private List<TimingHolder> invoiceHandlingTimesPerStateAndUser = null;

	private List<String> nonActiveStates;

	private int EXPIRING_THRESHOLD_DAYS = 7;
	private int URGENT_THRESHOLD_DAYS = 14;

	private String EXPIRED_LABEL = "Scadute";
	private String EXPIRING_LABEL = "In scadenza (< 7 gg)";
	private String URGENT_LABEL = "Urgenti (7-14 gg)";
	private String NON_URGENT_LABEL = "Non urgenti (> 14 gg)";
	public static final String ADMIN_ROLE= "AMMINISTRATORE";
	
	
	/**
	 * Variabile helper per tenere traccia del ruolo amministrativo di un utente
	 */
	protected Boolean isAdmin = null;
	
	public Boolean getIsAdmin() {
		isAdmin = false;
		if (isAdmin == null){
			Role r = JNDIUtils.getRoleManager().getRoleByDescription(ADMIN_ROLE);
			for (ProfileDetail pd : SessionUtils.getCurrentUserProfileDetail()) {
				if (pd.getRole().getRoleId() == r.getRoleId()) {
					isAdmin = true;
				}
			}
		}
		return isAdmin;
	}
	
	public Long getInvoiceNumber() {
		if ( (invoiceNumber == null) || (invoiceNumber < 0) ) {
			retrieveInvoiceNumber();
		}
		return invoiceNumber;
	}
	
	public Date getFirstDocument() {
		if (firstDocument== null) {
			retrieveFirstDocument();
		}
		return firstDocument;
	}

	private void retrieveFirstDocument() {
		List out = JNDIUtils.getCrudManager().findByNativeQuery("SELECT MIN(receiptDateTime) from OriginalDocuments");
		if ( (out != null) && (out.size() > 0 ) ){
			try {
				this.firstDocument = (Date) out.get(0);
			} catch (Exception e) {
				e.printStackTrace();
				this.firstDocument = new Date(0);
			}
		}
		else
			this.firstDocument = new Date(0);
		
		
	}
	private void retrieveInvoiceNumber() {
		this.invoiceNumber = new Long( (Long)JNDIUtils.getCrudManager().findByQuery("SELECT count (*) from Invoice").get(0) );
	}

	public List<InvoiceStatisticsHolder> getInvoiceStates() {
		if (invoiceStates == null) {
			retrieveInvoiceStates();
		}
		return invoiceStates;
	}
	
	public List<InvoiceStatisticsHolder> getEndedInvoiceStates() {
		if (endedInvoiceStates == null) {
			retrieveEndedInvoiceStates();
		}
		return endedInvoiceStates ;
	}

	public List<InvoiceStatisticsHolder> getExpiringInvoices() {
		if (expiringInvoices == null) {
			retrieveExpiringInvoices();
		}
		return expiringInvoices;
	}

	public List<TimingHolder> getInvoiceHandlingTimesPerState() {
		if (invoiceHandlingTimesPerState == null) {
			retrieveInvoiceHandlingTimesPerState();
		}
		return invoiceHandlingTimesPerState;
	}

	public List<TimingHolder> getInvoiceHandlingTimesPerStateAndUser() {
		if (invoiceHandlingTimesPerStateAndUser == null) {
			retrieveInvoiceHandlingTimesPerStateAndUser();
		}
		return invoiceHandlingTimesPerStateAndUser;
	}

	public List<String> getNonActiveStates() {
		return nonActiveStates;
	}

	public void setNonActiveStates(List<String> nonActiveStates) {
		this.nonActiveStates = nonActiveStates;
	}

	/**
	 * Tempi medi di elaborazione di fatture per stato
	 * 
	 * 
	 */
	
	private void retrieveInvoiceHandlingTimesPerState() {
		invoiceHandlingTimesPerState = new ArrayList<TimingHolder>();

		List<Object[]> out = JNDIUtils
				.getCrudManager()
				.findByNativeQuery(
						"SELECT AVG(TIME_TO_SEC(TIMEDIFF(endTime, startTime)))/(60*60), MAX(TIME_TO_SEC(TIMEDIFF(endTime, startTime)))/(60*60), i.description FROM TaskTimings t, InvoiceStates i where t.invoiceStateId=i.invoiceStateId group by i.invoiceStateId");
		if ((out != null) && (out.size() > 0) && (out.get(0) != null)) {
			for (Object[] object : out) {
				if ((object[0] != null) && (object[1] != null)
						&& (object[2] != null) ) {
					double averageTime = ((Number) object[0]).doubleValue();
					double maxTime = ((Number) object[1]).doubleValue();
					String state = (String) object[2];
					invoiceHandlingTimesPerState.add(new TimingHolder(
							"", state, averageTime, maxTime));
				}
			}

		}

	}
	
	/**
	 * Tempi medi e massimi di elaborazione di fatture per stato e utente
	 * 
	 * 
	 */
	private void retrieveInvoiceHandlingTimesPerStateAndUser() {
		invoiceHandlingTimesPerStateAndUser = new ArrayList<TimingHolder>();

		List<Object[]> out = JNDIUtils
				.getCrudManager()
				.findByNativeQuery(
						"SELECT AVG(TIME_TO_SEC(TIMEDIFF(endTime, startTime)))/(60*60), MAX(TIME_TO_SEC(TIMEDIFF(endTime, startTime)))/(60*60) , t.username, i.description FROM TaskTimings t, InvoiceStates i where t.invoiceStateId=i.invoiceStateId group by t.username, i.invoiceStateId");
		if ((out != null) && (out.size() > 0) && (out.get(0) != null)) {
			for (Object[] object : out) {
				if ((object[0] != null) && (object[1] != null)
						&& (object[2] != null) && (object[3] != null)) {
					double averageTime = ((Number) object[0]).doubleValue();
					double maxTime = ((Number) object[1]).doubleValue();
					String username = (String) object[2];
					String state = (String) object[3];
					invoiceHandlingTimesPerStateAndUser.add(new TimingHolder(
							username, state, averageTime, maxTime));
				}
			}

		}

	}

	private void retrieveExpiringInvoices() {
		expiringInvoices = new ArrayList<InvoiceStatisticsHolder>();

		GregorianCalendar today = new GregorianCalendar();
		today.setTime(new Date());
		today.set(Calendar.HOUR_OF_DAY, 0);
		today.set(Calendar.MINUTE, 1);

		GregorianCalendar expiring_limit = new GregorianCalendar();
		expiring_limit.setTime(new Date());
		expiring_limit.add(Calendar.DAY_OF_YEAR, EXPIRING_THRESHOLD_DAYS);
		expiring_limit.set(Calendar.HOUR_OF_DAY, 0);
		expiring_limit.set(Calendar.MINUTE, 1);

		GregorianCalendar urgent_limit = new GregorianCalendar();
		urgent_limit.setTime(new Date());
		urgent_limit.add(Calendar.DAY_OF_YEAR, URGENT_THRESHOLD_DAYS);
		urgent_limit.set(Calendar.HOUR_OF_DAY, 0);
		urgent_limit.set(Calendar.MINUTE, 1);

		String queryExpired = "SELECT COUNT (*) FROM Invoice i WHERE paymentDetail.paymentDueDate < :startPaymentDueDate and i.invoiceState.description NOT IN "+getNonActiveStatesAsSQLString();
		String queryExpiring = "SELECT COUNT (*) FROM Invoice i WHERE paymentDetail.paymentDueDate >= :startPaymentDueDate and paymentDetail.paymentDueDate <= :endPaymentDueDate and i.invoiceState.description NOT IN "+getNonActiveStatesAsSQLString();
		String queryUrgent = "SELECT COUNT (*) FROM Invoice i WHERE paymentDetail.paymentDueDate >= :startPaymentDueDate and paymentDetail.paymentDueDate < :endPaymentDueDate and i.invoiceState.description NOT IN "+getNonActiveStatesAsSQLString();
		String queryNonUrgent = "SELECT COUNT (*) FROM Invoice i WHERE paymentDetail.paymentDueDate > :startPaymentDueDate and i.invoiceState.description NOT IN "+getNonActiveStatesAsSQLString();

		//TODO: sistemare intervalli: fatture che cascano nei limiti degli intervalli vengono conteggiate due volte...

		System.out.println("Today:"+today.getTime().toString());
		System.out.println("Expiring limit:"+expiring_limit.getTime().toString());
		System.out.println("Urgent limit:"+urgent_limit.getTime().toString());
		
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("startPaymentDueDate", DateUtils.endOfDay(today.getTime()));

		long expired = (Long) JNDIUtils.getCrudManager().findByQuery(queryExpired,
				params).get(0);

		
		params = new HashMap<String, Object>();
		params.put("startPaymentDueDate", DateUtils.beginOfDay(today.getTime()));
		params.put("endPaymentDueDate", DateUtils.endOfDay(expiring_limit.getTime()));

		long expiring = (Long) JNDIUtils.getCrudManager().findByQuery(queryExpiring,
				params).get(0);
		
		params = new HashMap<String, Object>();
		params.put("startPaymentDueDate", DateUtils.beginOfDay(expiring_limit.getTime()));
		params.put("endPaymentDueDate", DateUtils.endOfDay(urgent_limit.getTime()));

		long urgent = (Long) JNDIUtils.getCrudManager().findByQuery(queryUrgent,
				params).get(0);

		params = new HashMap<String, Object>();
		params.put("startPaymentDueDate", DateUtils.beginOfDay(urgent_limit.getTime()));
		long nonUrgent = (Long) JNDIUtils.getCrudManager().findByQuery(queryNonUrgent,
				params).get(0);


		expiringInvoices
				.add(new InvoiceStatisticsHolder(EXPIRED_LABEL, expired));
		expiringInvoices.add(new InvoiceStatisticsHolder(EXPIRING_LABEL,
				expiring));
		expiringInvoices.add(new InvoiceStatisticsHolder(URGENT_LABEL, urgent));
		expiringInvoices.add(new InvoiceStatisticsHolder(NON_URGENT_LABEL,
				nonUrgent));

	}

	private String getNonActiveStatesAsSQLString() {
		List<String> states = getNonActiveStates();
		if ( (states == null) || (states.size() == 0) )
			return "()";
		else{
			String out = "(";
			for (String state : states) {
				out+="'"+state+"'"+", ";
			}
			out = out.substring(0, out.length() -2);
			out +=")";
			return out;
		}
	}

	private void retrieveInvoiceStates() {
		List<InvoiceState> states = JNDIUtils.getCrudManager().findByQuery(
				"from InvoiceState");
		invoiceStates = new ArrayList<InvoiceStatisticsHolder>();
		if (states != null)
			for (InvoiceState invoiceState : states) {
				if ( ( nonActiveStates != null) && ( nonActiveStates.contains(invoiceState.getDescription())) ){
					continue;//salto gli stati considerati non attivi
				}
				else{
					int currentStateCount = JNDIUtils.getCrudManager()
							.countByCriteria(
									Invoice.class,
									Arrays.asList((Criterion) new NestedCriterion(
											"invoiceState", Restrictions.eq(
													"description", invoiceState
															.getDescription()))));
	
					invoiceStates.add(new InvoiceStatisticsHolder(invoiceState
							.getDescription(), currentStateCount));
				}
			}

	}
	
	private void retrieveEndedInvoiceStates() {
		List<InvoiceState> states = JNDIUtils.getCrudManager().findByQuery(
				"from InvoiceState");
		endedInvoiceStates = new ArrayList<InvoiceStatisticsHolder>();
		if (states != null)
			for (InvoiceState invoiceState : states) {
				if ( ( nonActiveStates != null) && ( nonActiveStates.contains(invoiceState.getDescription())) ){
				
					int currentStateCount = JNDIUtils.getCrudManager()
							.countByCriteria(
									Invoice.class,
									Arrays.asList((Criterion) new NestedCriterion(
											"invoiceState", Restrictions.eq(
													"description", invoiceState
															.getDescription()))));
	
					endedInvoiceStates.add(new InvoiceStatisticsHolder(invoiceState
							.getDescription(), currentStateCount));
				}
			}

	}

	public void refreshStatistics() {
		this.invoiceStates = null;
		this.endedInvoiceStates = null;
		this.expiringInvoices = null;
		this.firstDocument = null;
		this.invoiceNumber = null;
		this.invoiceHandlingTimesPerState = null;
		this.invoiceHandlingTimesPerStateAndUser = null;
	}

}
